
'Utilites for Cognite API SDK\n\nThis module provides helper methods and different utilities for the Cognite API Python SDK.\n\nThis module is protected and should not used by end-users.\n'
import gzip
import json
import re
import sys
import time
from datetime import datetime
import requests
import cognite.config as config

def delete_request(url, params=None, headers=None, cookies=None):
    'Perform a DELETE request with a predetermined number of retries.'
    for number_of_tries in range((config.get_number_of_retries() + 1)):
        try:
            res = requests.delete(url, params=params, headers=headers, cookies=cookies)
            if (res.status_code == 200):
                return res
        except Exception as e:
            if (number_of_tries == config.get_number_of_retries()):
                raise APIError(e)
    try:
        err_mess = res.json()['error'].__str__()
    except:
        err_mess = res.content.__str__()
    err_mess += '\nX-Request_id: {}'.format(res.headers.get('X-Request-Id'))
    raise APIError(err_mess)

def get_request(url, params=None, headers=None, cookies=None):
    'Perform a GET request with a predetermined number of retries.'
    for number_of_tries in range((config.get_number_of_retries() + 1)):
        try:
            res = requests.get(url, params=params, headers=headers, cookies=cookies)
            if (res.status_code == 200):
                return res
        except Exception as e:
            if (number_of_tries == config.get_number_of_retries()):
                raise APIError(e)
    try:
        err_mess = res.json()['error'].__str__()
    except:
        err_mess = res.content.__str__()
    err_mess += '\nX-Request_id: {}'.format(res.headers.get('X-Request-Id'))
    raise APIError(err_mess)

def post_request(url, body, headers=None, params=None, cookies=None, use_gzip=False):
    'Perform a POST request with a predetermined number of retries.'
    for number_of_tries in range((config.get_number_of_retries() + 1)):
        try:
            if use_gzip:
                if headers:
                    headers['Content-Encoding'] = 'gzip'
                else:
                    headers = {'Content-Encoding': 'gzip'}
                res = requests.post(url, data=gzip.compress(json.dumps(body).encode('utf-8')), headers=headers, params=params, cookies=cookies)
            else:
                res = requests.post(url, data=json.dumps(body), headers=headers, params=params, cookies=cookies)
            if (res.status_code == 200):
                return res
        except Exception as e:
            if (number_of_tries == config.get_number_of_retries()):
                raise APIError(e)
    try:
        err_mess = res.json()['error'].__str__()
    except:
        err_mess = res.content.__str__()
    err_mess += '\nX-Request_id: {}'.format(res.headers.get('X-Request-Id'))
    raise APIError(err_mess)

def put_request(url, body=None, headers=None, cookies=None):
    'Perform a PUT request with a predetermined number of retries.'
    for number_of_tries in range((config.get_number_of_retries() + 1)):
        try:
            res = requests.put(url, data=json.dumps(body), headers=headers, cookies=cookies)
            if res.ok:
                return res
        except Exception as e:
            if (number_of_tries == config.get_number_of_retries()):
                raise APIError(e)
    try:
        err_mess = res.json()['error'].__str__()
    except:
        err_mess = res.content.__str__()
    err_mess += '\nX-Request_id: {}'.format(res.headers.get('X-Request-Id'))
    raise APIError(err_mess)

def datetime_to_ms(dt):
    epoch = datetime.utcfromtimestamp(0)
    return int(((dt - epoch).total_seconds() * 1000))

def round_to_nearest(x, base):
    return int((base * round((float(x) / base))))

def granularity_to_ms(time_string):
    'Returns millisecond representation of granularity time string'
    magnitude = int(''.join([c for c in time_string if c.isdigit()]))
    unit = ''.join([c for c in time_string if c.isalpha()])
    unit_in_ms = {'s': 1000, 'second': 1000, 'm': 60000, 'minute': 60000, 'h': 3600000, 'hour': 3600000, 'd': 86400000, 'day': 86400000}
    return (magnitude * unit_in_ms[unit])

def _time_ago_to_ms(time_ago_string):
    'Returns millisecond representation of time-ago string'
    pattern = '(\\d+)([a-z])-ago'
    res = re.match(pattern, str(time_ago_string))
    if res:
        magnitude = int(res.group(1))
        unit = res.group(2)
        unit_in_ms = {'s': 1000, 'm': 60000, 'h': 3600000, 'd': 86400000, 'w': 604800000}
        return (magnitude * unit_in_ms[unit])
    return None

def interval_to_ms(start, end):
    'Returns the ms representation of start-end-interval whether it is time-ago, datetime or None.'
    time_now = int(round((time.time() * 1000)))
    if isinstance(start, datetime):
        start = datetime_to_ms(start)
    elif isinstance(start, str):
        start = (time_now - _time_ago_to_ms(start))
    elif (start is None):
        start = (time_now - _time_ago_to_ms('2w-ago'))
    if isinstance(end, datetime):
        end = datetime_to_ms(end)
    elif isinstance(end, str):
        end = (time_now - _time_ago_to_ms(end))
    elif (end is None):
        end = time_now
    return (start, end)

class APIError(Exception):
    pass

class ProgressIndicator():
    'This class lets the system give the user that data is being downloaded'

    def __init__(self, tag_ids):
        sys.stdout.write('\rDownloading data for tags {}...'.format(tag_ids))

    def terminate(self):
        sys.stdout.write((('\r' + (' ' * 500)) + '\r'))

class Bin():
    '\n    Attributes:\n        entries (List): List of entries.\n        get_count (Callable): Callable function to get count.\n    '

    def __init__(self, get_count):
        '\n        Args:\n            get_count: A function that will take an element and get the count of something in it.\n        '
        self.entries = []
        self.get_count = get_count

    def add_item(self, item):
        self.entries.append(item)

    def sum(self):
        total = 0
        for elem in self.entries:
            total += self.get_count(elem)
        return total

    def show(self):
        return self.entries

def first_fit(list_items, max_size, get_count):
    'Returns list of bins with input items inside.'
    list_items = sorted(list_items, key=get_count, reverse=True)
    list_bins = [Bin(get_count=get_count)]
    for item in list_items:
        alloc_flag = False
        for bin in list_bins:
            if ((bin.sum() + get_count(item)) <= max_size):
                bin.add_item(item)
                alloc_flag = True
                break
        if (not alloc_flag):
            new_bin = Bin(get_count=get_count)
            new_bin.add_item(item)
            list_bins.append(new_bin)
    list_items = []
    for bin in list_bins:
        list_items.append(bin.show())
    return list_items



import os

import numpy as np
import pandas as pd
import pytest

from cognite.v05 import files


def test_upload_file_metadata():
    response = files.upload_file('test_file', source='sdk-tests', overwrite=True)
    assert response.get('uploadURL') is not None
    assert response.get('fileId') is not None


def test_upload_file(tmpdir):
    file_path = os.path.join(tmpdir, 'test_file.txt')
    tmpdir.join('test_file.txt').write("This is a test file.")
    with pytest.warns(UserWarning):
        response = files.upload_file('test_file', file_path, source='sdk-tests', overwrite=True)
    assert response.get('uploadURL') is None
    assert response.get('fileId') is not None


def test_list_files():
    from cognite.v05.dto import FileListResponse
    response = files.list_files(limit=3)
    assert isinstance(response, FileListResponse)
    assert isinstance(response.to_pandas(), pd.DataFrame)
    assert isinstance(response.to_json(), list)
    assert isinstance(response.to_ndarray(), np.ndarray)
    assert len(response.to_json()) > 0 and len(response.to_json()) <= 3


def test_list_files_empty():
    response = files.list_files(source='not_a_source')
    assert response.to_pandas().empty
    assert len(response.to_json()) == 0


@pytest.fixture(scope='module')
def file_id():
    res = files.list_files(name='test_file', source='sdk-tests', limit=1)
    return res.to_json()[0]['id']


def test_get_file_info(file_id):
    from cognite.v05.dto import FileInfoResponse
    response = files.get_file_info(file_id)
    assert isinstance(response, FileInfoResponse)
    assert isinstance(response.to_json(), dict)
    assert isinstance(response.to_ndarray(), np.ndarray)
    assert isinstance(response.to_pandas(), pd.DataFrame)
    assert response.id == file_id


@pytest.mark.parametrize('get_contents', [True, False])
def test_download_files(file_id, get_contents):
    try:
        response = files.download_file(file_id, get_contents)
        if get_contents:
            assert isinstance(response, bytes)
        else:
            assert isinstance(response, str)
    except Exception as e:
        print("Failed to download file: ", e)


def test_delete_file(file_id):
    response = files.delete_files([file_id])
    assert file_id in response['deleted'] or file_id in response['failed']




'Raw Module\n\nThis module mirrors the Raw API. It allows the user to handle raw data.\n\nhttps://doc.cognitedata.com/0.5/#Cognite-API-Cloud-Raw\n'
import cognite._utils as _utils
import cognite.config as config
from cognite.v05.dto import RawResponse, RawRow

def get_databases(limit=None, cursor=None, api_key=None, project=None):
    'Returns a RawObject containing a list of raw databases.\n\n    Args:\n        limit (int):    A limit on the amount of results to return.\n\n        cursor (str):   A cursor can be provided to navigate through pages of results.\n\n        api_key (str):  Your api-key.\n\n        project (str):  Project name.\n\n    Returns:\n        v05.dto.RawResponse: A data object containing the requested data with several getter methods with different\n        output formats.\n    '
    (api_key, project) = config.get_config_variables(api_key, project)
    url = (config.get_base_url(api_version=0.5) + '/projects/{}/raw'.format(project))
    params = {'limit': limit, 'cursor': cursor}
    headers = {'api-key': api_key, 'content-type': '*/*', 'accept': 'application/json'}
    res = _utils.get_request(url=url, params=params, headers=headers, cookies=config.get_cookies())
    return RawResponse(res.json())

def create_databases(database_names, api_key=None, project=None):
    'Creates databases in the Raw API and returns the created databases.\n\n    Args:\n        database_names (list):  A list of databases to create.\n\n        api_key (str):          Your api-key.\n\n        project (str):          Project name.\n\n    Returns:\n        v05.dto.RawResponse: A data object containing the requested data with several getter methods with different\n        output formats.\n\n    '
    (api_key, project) = config.get_config_variables(api_key, project)
    url = (config.get_base_url(api_version=0.5) + '/projects/{}/raw/create'.format(project))
    body = {'items': [{'dbName': '{}'.format(database_name)} for database_name in database_names]}
    headers = {'api-key': api_key, 'content-type': '*/*', 'accept': 'application/json'}
    res = _utils.post_request(url=url, body=body, headers=headers, cookies=config.get_cookies())
    return RawResponse(res.json())

def delete_databases(database_names, recursive=False, api_key=None, project=None):
    'Deletes databases in the Raw API.\n\n    Args:\n        database_names (list):  A list of databases to delete.\n\n        api_key (str):          Your api-key.\n\n        project (str):          Project name.\n\n    Returns:\n        An empty response.\n\n    '
    (api_key, project) = config.get_config_variables(api_key, project)
    url = (config.get_base_url(api_version=0.5) + '/projects/{}/raw/delete'.format(project))
    body = {'items': [{'dbName': '{}'.format(database_name)} for database_name in database_names]}
    params = {'recursive': recursive}
    headers = {'api-key': api_key, 'content-type': '*/*', 'accept': 'application/json'}
    res = _utils.post_request(url=url, body=body, params=params, headers=headers, cookies=config.get_cookies())
    return res.json()

def get_tables(database_name=None, limit=None, cursor=None, api_key=None, project=None):
    'Returns a RawObject containing a list of tables in a raw database.\n\n    Args:\n        database_name (str):   The database name to retrieve tables from.\n\n        limit (int):    A limit on the amount of results to return.\n\n        cursor (str):   A cursor can be provided to navigate through pages of results.\n\n        api_key (str):  Your api-key.\n\n        project (str):  Project name.\n\n    Returns:\n        v05.dto.RawResponse: A data object containing the requested data with several getter methods with different\n        output formats.\n    '
    (api_key, project) = config.get_config_variables(api_key, project)
    url = (config.get_base_url(api_version=0.5) + '/projects/{}/raw/{}'.format(project, database_name))
    params = dict()
    if (not limit):
        params['limit'] = limit
    if (not cursor):
        params['cursor'] = cursor
    headers = {'api-key': api_key, 'content-type': '*/*', 'accept': 'application/json'}
    res = _utils.get_request(url=url, params=params, headers=headers, cookies=config.get_cookies())
    return RawResponse(res.json())

def create_tables(database_name=None, table_names=None, api_key=None, project=None):
    'Creates tables in the given Raw API database.\n\n    Args:\n        database_name (str):    The database to create tables in.\n\n        table_names (list):     The table names to create.\n\n        api_key (str):          Your api-key.\n\n        project (str):          Project name.\n\n    Returns:\n        v05.dto.RawResponse: A data object containing the requested data with several getter methods with different\n        output formats.\n\n    '
    (api_key, project) = config.get_config_variables(api_key, project)
    url = (config.get_base_url(api_version=0.5) + '/projects/{}/raw/{}/create'.format(project, database_name))
    body = {'items': [{'tableName': '{}'.format(table_name)} for table_name in table_names]}
    headers = {'api-key': api_key, 'content-type': '*/*', 'accept': 'application/json'}
    res = _utils.post_request(url=url, body=body, headers=headers, cookies=config.get_cookies())
    return RawResponse(res.json())

def delete_tables(database_name=None, table_names=None, api_key=None, project=None):
    'Deletes databases in the Raw API.\n\n    Args:\n        database_name (str):    The database to create tables in.\n\n        table_names (list):     The table names to create.\n\n        api_key (str):          Your api-key.\n\n        project (str):          Project name.\n\n    Returns:\n        An empty response.\n\n    '
    (api_key, project) = config.get_config_variables(api_key, project)
    url = (config.get_base_url(api_version=0.5) + '/projects/{}/raw/{}/delete'.format(project, database_name))
    body = {'items': [{'tableName': '{}'.format(table_name)} for table_name in table_names]}
    headers = {'api-key': api_key, 'content-type': '*/*', 'accept': 'application/json'}
    res = _utils.post_request(url=url, body=body, headers=headers, cookies=config.get_cookies())
    return res.json()

def get_rows(database_name=None, table_name=None, limit=None, cursor=None, api_key=None, project=None):
    'Returns a RawObject containing a list of rows.\n\n    Args:\n        database_name (str):    The database name to retrieve rows from.\n\n        table_name (str):       The table name to retrieve rows from.\n\n        limit (int):            A limit on the amount of results to return.\n\n        cursor (str):           A cursor can be provided to navigate through pages of results.\n\n        api_key (str):          Your api-key.\n\n        project (str):          Project name.\n\n    Returns:\n        v05.dto.RawResponse: A data object containing the requested data with several getter methods with different\n        output formats.\n    '
    (api_key, project) = config.get_config_variables(api_key, project)
    url = (config.get_base_url(api_version=0.5) + '/projects/{}/raw/{}/{}'.format(project, database_name, table_name))
    params = dict()
    params['limit'] = limit
    params['cursor'] = cursor
    headers = {'api-key': api_key, 'content-type': '*/*', 'accept': 'application/json'}
    res = _utils.get_request(url=url, params=params, headers=headers, cookies=config.get_cookies())
    return RawResponse(res.json())

def create_rows(database_name=None, table_name=None, rows=None, api_key=None, project=None, ensure_parent=False, use_gzip=False):
    "Creates tables in the given Raw API database.\n\n    Args:\n        database_name (str):    The database to create rows in.\n\n        table_name (str):       The table names to create rows in.\n\n        rows (list[v05.dto.RawRow]):            The rows to create.\n\n        api_key (str):          Your api-key.\n\n        project (str):          Project name.\n\n        ensure_parent (bool):   Create database/table if it doesn't exist already\n\n        use_gzip (bool):        Compress content using gzip\n\n    Returns:\n        An empty response\n\n    "
    (api_key, project) = config.get_config_variables(api_key, project)
    url = (config.get_base_url(api_version=0.5) + '/projects/{}/raw/{}/{}/create'.format(project, database_name, table_name))
    headers = {'api-key': api_key, 'content-type': '*/*', 'accept': 'application/json'}
    if ensure_parent:
        params = {'ensureParent': 'true'}
    else:
        params = {}
    ul_row_limit = 1000
    i = 0
    while (i < len(rows)):
        body = {'items': [{'key': '{}'.format(row.key), 'columns': row.columns} for row in rows[i:(i + ul_row_limit)]]}
        res = _utils.post_request(url=url, body=body, headers=headers, params=params, cookies=config.get_cookies(), use_gzip=use_gzip)
        i += ul_row_limit
    return res.json()

def delete_rows(database_name=None, table_name=None, rows=None, api_key=None, project=None):
    'Deletes rows in the Raw API.\n\n    Args:\n        database_name (str):    The database to create tables in.\n\n        table_name (str):      The table name where the rows are at.\n\n        rows (list):            The rows to delete.\n\n        api_key (str):          Your api-key.\n\n        project (str):          Project name.\n\n    Returns:\n        An empty response.\n\n    '
    (api_key, project) = config.get_config_variables(api_key, project)
    url = (config.get_base_url(api_version=0.5) + '/projects/{}/raw/{}/{}/delete'.format(project, database_name, table_name))
    body = {'items': [{'key': '{}'.format(row.key), 'columns': row.columns} for row in rows]}
    headers = {'api-key': api_key, 'content-type': '*/*', 'accept': 'application/json'}
    res = _utils.post_request(url=url, body=body, headers=headers, cookies=config.get_cookies())
    return res.json()

def get_row(database_name=None, table_name=None, row_key=None, api_key=None, project=None):
    'Returns a RawObject containing a list of rows.\n\n    Args:\n        database_name (str):    The database name to retrieve rows from.\n\n        table_name (str):       The table name to retrieve rows from.\n\n        row_key (str):          The key of the row to fetch.\n\n        api_key (str):          Your api-key.\n\n        project (str):          Project name.\n\n    Returns:\n        v05.dto.RawResponse: A data object containing the requested data with several getter methods with different\n        output formats.\n    '
    (api_key, project) = config.get_config_variables(api_key, project)
    url = (config.get_base_url(api_version=0.5) + '/projects/{}/raw/{}/{}/{}'.format(project, database_name, table_name, row_key))
    params = dict()
    headers = {'api-key': api_key, 'content-type': '*/*', 'accept': 'application/json'}
    res = _utils.get_request(url=url, params=params, headers=headers, cookies=config.get_cookies())
    return RawResponse(res.json())




'Tag Matching Module\n\nThis module mirrors the Tag Matching API. It allows the user to search for tag id matches.\n\nhttps://doc.cognitedata.com/0.5/#Cognite-API-Tag-Matching\n'
import cognite._utils as _utils
import cognite.config as config
from cognite.v05.dto import TagMatchingResponse

def tag_matching(tag_ids, fuzzy_threshold=0, platform=None, **kwargs):
    'Returns a TagMatchingObject containing a list of matched tags for the given query.\n\n    This method takes an arbitrary string as argument and performs fuzzy matching with a user defined threshold\n    toward tag ids in the system.\n\n    Args:\n        tag_ids (list):         The tag_ids to retrieve matches for.\n\n        fuzzy_threshold (int):  The threshold to use when searching for matches. A fuzzy threshold of 0 means you only\n                                want to accept perfect matches. Must be >= 0.\n\n        platform (str):         The platform to search on.\n\n    Keyword Args:\n        api_key (str):          Your api-key.\n\n        project (str):          Project name.\n\n    Returns:\n        v05.dto.TagMatchingResponse: A data object containing the requested data with several getter methods with different\n        output formats.\n    '
    (api_key, project) = config.get_config_variables(kwargs.get('api_key'), kwargs.get('project'))
    url = (config.get_base_url(api_version=0.5) + '/projects/{}/tagmatching'.format(project))
    body = {'tagIds': tag_ids, 'metadata': {'fuzzyThreshold': fuzzy_threshold, 'platform': platform}}
    headers = {'api-key': api_key, 'content-type': '*/*', 'accept': 'application/json'}
    res = _utils.post_request(url=url, body=body, headers=headers, cookies=config.get_cookies())
    return TagMatchingResponse(res.json())



# Temporary mock setup while waiting for a better way to do integration tests
from unittest.mock import patch

import pytest

from tests.conftest import MockReturnValue


@patch('requests.post')
@pytest.fixture(scope='module')
def tagmatching_result(mock_post):
    from cognite.v05.tagmatching import tag_matching
    response = {'data': {'items': [{'matches': [
        {'platform': 'a_platform', 'score': 0, 'tagId': 'a_match'},
        {'platform': 'a_platform', 'score': 0, 'tagId': 'a_match1'},
        {'platform': 'a_platform', 'score': 0, 'tagId': 'a_match2'}], 'tagId': 'a_tag'}]}}
    mock_post.return_value = MockReturnValue(status=200, json_data=response)
    return tag_matching(tag_ids=['a_tag'])


def test_object(tagmatching_result):
    from cognite.v05.dto import TagMatchingResponse
    assert isinstance(tagmatching_result, TagMatchingResponse)


def test_json(tagmatching_result):
    assert isinstance(tagmatching_result.to_json(), list)


def test_pandas(tagmatching_result):
    import pandas as pd
    assert isinstance(tagmatching_result.to_pandas(), pd.DataFrame)


def test_ndarray(tagmatching_result):
    import numpy as np
    assert isinstance(tagmatching_result.to_ndarray(), np.ndarray)


def test_list_len_first_matches(tagmatching_result):
    l = tagmatching_result.to_list(first_matches_only=False)
    assert len(l) == 3


def test_list_len_all_matches(tagmatching_result):
    l = tagmatching_result.to_list()
    assert len(l) == 1




'Data Objects\n\nThis module contains data objects used to represent the data returned from the API. These objects have at least\nthe following output formats:\n\n    * to_pandas():    Returns pandas dataframe\n    * to_ndarray():   Numpy array\n    * to_json():      Json format\n'
import abc
import json
from copy import deepcopy
import pandas as pd
import six
from cognite import _utils

@six.add_metaclass(abc.ABCMeta)
class CogniteDataObject():
    'Abstract Cognite Data Object\n\n    This abstract class provides a skeleton for all data objects in this module. All data objects should inherit\n    this class.\n    '

    def __init__(self, internal_representation):
        self.internal_representation = internal_representation

    @abc.abstractmethod
    def to_pandas(self):
        'Returns data as a pandas dataframe'
        pass

    @abc.abstractmethod
    def to_json(self):
        'Returns data as a json object'
        pass

    def to_ndarray(self):
        'Returns data as a numpy array'
        return self.to_pandas().values

    def next_cursor(self):
        'Returns next cursor to use for paging through results'
        if self.internal_representation.get('data'):
            return self.internal_representation.get('data').get('nextCursor')

    def previous_cursor(self):
        'Returns previous cursor'
        if self.internal_representation.get('data'):
            return self.internal_representation.get('data').get('previousCursor')

class RawResponse(CogniteDataObject):
    'Raw Response Object.'

    def to_json(self):
        'Returns data as a json object'
        return self.internal_representation['data']['items']

    def to_pandas(self):
        'Returns data as a pandas dataframe'
        return pd.DataFrame(self.internal_representation['data']['items'])

class RawRow(object):
    'DTO for a row in a raw database.\n\n    The Raw API is a simple key/value-store. Each row in a table in a raw database consists of a\n    unique row key and a set of columns.\n\n    Attributes:\n        key (str):      Unique key for the row.\n\n        columns (int):  A key/value-map consisting of the values in the row.\n    '

    def __init__(self, key, columns):
        self.key = key
        self.columns = columns

    def __repr__(self):
        return json.dumps(self.repr_json())

    def repr_json(self):
        return self.__dict__

class TagMatchingResponse(CogniteDataObject):
    'Tag Matching Response Object.\n\n    In addition to the standard output formats this data object also has a to_list() method which returns a list of\n    names of the tag matches.\n    '

    def to_pandas(self):
        'Returns data as a pandas dataframe'
        matches = []
        for tag in self.internal_representation['data']['items']:
            for match in tag['matches']:
                matches.append({'tag': tag['tagId'], 'match': match['tagId'], 'score': match['score'], 'platform': match['platform']})
        if matches:
            return pd.DataFrame(matches)[['tag', 'match', 'platform', 'score']]
        return pd.DataFrame()

    def to_json(self):
        'Returns data as a json object'
        return self.internal_representation['data']['items']

    def to_list(self, first_matches_only=True):
        'Returns a list representation of the matches.\n\n        Args:\n            first_matches_only (bool):      Boolean determining whether or not to return only the top match for each\n                                            tag.\n\n        Returns:\n            list: list of matched tags.\n        '
        if self.to_pandas().empty:
            return []
        if first_matches_only:
            return self.to_pandas().sort_values(['score', 'match']).groupby(['tag']).first()['match'].tolist()
        return self.to_pandas().sort_values(['score', 'match'])['match'].tolist()

class DatapointsQuery():
    "Data Query Object for Datapoints.\n\n    Attributes:\n        name (str):           Unique name of the time series.\n        aggregates (list):          The aggregate functions to be returned. Use default if null. An empty string must\n                                    be sent to get raw data if the default is a set of aggregate functions.\n        granularity (str):          The granularity size and granularity of the aggregates.\n        start (str, int, datetime): Get datapoints after this time. Format is N[timeunit]-ago where timeunit is w,d,h,m,s.\n                                    Example: '2d-ago' will get everything that is up to 2 days old. Can also send time in\n                                    ms since epoch or as a datetime object.\n        end (str, int, datetime):   Get datapoints up to this time. The format is the same as for start.\n    "

    def __init__(self, name, aggregates=None, granularity=None, start=None, end=None, limit=None):
        self.name = name
        self.aggregates = (','.join(aggregates) if (aggregates is not None) else None)
        self.granularity = granularity
        (self.start, self.end) = _utils.interval_to_ms(start, end)
        if (not start):
            self.start = None
        if (not end):
            self.end = None
        self.limit = limit

class DatapointsResponse(CogniteDataObject):
    'Datapoints Response Object.'

    def to_json(self):
        'Returns data as a json object'
        return self.internal_representation['data']['items'][0]

    def to_pandas(self):
        'Returns data as a pandas dataframe'
        return pd.DataFrame(self.internal_representation['data']['items'][0]['datapoints'])

class DatapointsResponseIterator():
    'Iterator for Datapoints Response Objects.'

    def __init__(self, datapoints_objects):
        self.datapoints_objects = datapoints_objects
        self.counter = 0

    def __iter__(self):
        return self

    def __next__(self):
        if (self.counter > (len(self.datapoints_objects) - 1)):
            raise StopIteration
        else:
            self.counter += 1
            return self.datapoints_objects[(self.counter - 1)]

class DatapointDepth(object):
    'Data transfer object for Depth datapoints.\n\n       Attributes:\n           depth (double): The depth (in m) of the datapoint \n           value (string):     The data value, Can be string or numeric depending on the metric.\n       '

    def __init__(self, depth, value):
        self.depth = depth
        self.value = value

class Datapoint(object):
    'Data transfer object for datapoints.\n\n    Attributes:\n        timestamp (int, datetime): The data timestamp in milliseconds since the epoch (Jan 1, 1970) or as a datetime object.\n        value (string):     The data value, Can be string or numeric depending on the metric.\n    '

    def __init__(self, timestamp, value):
        self.timestamp = (timestamp if isinstance(timestamp, int) else _utils.datetime_to_ms(timestamp))
        self.value = value

class TimeseriesWithDatapoints(object):
    'Data transfer object for a timeseries with datapoints.\n\n    Attributes:\n        tag_id (str):       Unique ID of time series.\n        datapoints (List[Datapoint]): List of datapoints in the timeseries.\n    '

    def __init__(self, name, datapoints):
        self.name = name
        self.datapoints = datapoints

class LatestDatapointResponse(CogniteDataObject):
    'Latest Datapoint Response Object.'

    def to_json(self):
        'Returns data as a json object'
        return self.internal_representation['data']['items'][0]

    def to_pandas(self):
        'Returns data as a pandas dataframe'
        return pd.DataFrame([self.internal_representation['data']['items'][0]])

    def to_ndarray(self):
        'Returns data as a numpy array'
        return self.to_pandas().values[0]

class TimeSeriesResponse(CogniteDataObject):
    'Time series Response Object'

    def to_json(self):
        'Returns data as a json object'
        return self.internal_representation['data']['items']

    def to_pandas(self):
        'Returns data as a pandas dataframe'
        items = deepcopy(self.internal_representation['data']['items'])
        if (items and (items[0].get('metadata') is None)):
            return pd.DataFrame(items)
        for d in items:
            if d.get('metadata'):
                d.update(d.pop('metadata'))
        return pd.DataFrame(items)

class TimeSeries(object):
    'Data Transfer Object for a time series.\n\n    Attributes:\n        name (str):       Unique name of time series.\n        is_string (bool):    Whether the time series is string valued or not.\n        metadata (dict):    Metadata.\n        unit (str):         Physical unit of the time series.\n        asset_id (str):     Asset that this time series belongs to.\n        description (str):  Description of the time series.\n        securityCategories (list(int)): Security categories required in order to access this time series.\n        step (bool):        Whether or not the time series is a step series.\n\n    '

    def __init__(self, name, is_string=False, metadata=None, unit=None, asset_id=None, description=None, security_categories=None, step=None):
        self.name = name
        self.isString = is_string
        self.metadata = metadata
        self.unit = unit
        self.assetId = asset_id
        self.description = description
        self.security_categories = security_categories
        self.step = step

class AssetListResponse(CogniteDataObject):
    'Assets Response Object'

    def __init__(self, internal_representation):
        super().__init__(internal_representation)
        self.counter = 0

    def to_json(self):
        'Returns data as a json object'
        return self.internal_representation['data']['items']

    def to_pandas(self):
        'Returns data as a pandas dataframe'
        if (len(self.to_json()) > 0):
            return pd.DataFrame(self.internal_representation['data']['items'])
        return pd.DataFrame()

    def __iter__(self):
        return self

    def __next__(self):
        if (self.counter > (len(self.to_json()) - 1)):
            raise StopIteration
        else:
            self.counter += 1
            return AssetResponse({'data': {'items': [self.to_json()[(self.counter - 1)]]}})

class AssetResponse(CogniteDataObject):

    def to_json(self):
        'Returns data as a json object'
        return self.internal_representation['data']['items'][0]

    def to_pandas(self):
        'Returns data as a pandas dataframe'
        if (len(self.to_json()) > 0):
            return pd.DataFrame.from_dict(self.to_json(), orient='index')
        return pd.DataFrame()

class Asset(object):
    'Data transfer object for assets.\n\n    Attributes:\n        name (str):                 Name of asset. Often referred to as tag.\n        parent_id (int):            ID of parent asset, if any.\n        description (str):          Description of asset.\n        metadata (dict):            Custom , application specific metadata. String key -> String Value.\n        ref_id (str):               Reference ID used only in post request to disambiguate references to duplicate\n                                    names.\n        parent_name (str):          Name of parent, this parent must exist in the same POST request.\n        parent_ref_id (list(int)):  Reference ID of parent, to disambiguate if multiple nodes have the same name.\n    '

    def __init__(self, name, parent_id=None, description=None, metadata=None, ref_id=None, parent_name=None, parent_ref_id=None):
        self.name = name
        self.parentId = parent_id
        self.description = description
        self.metadata = metadata
        self.refId = ref_id
        self.parentName = parent_name
        self.parentRefId = parent_ref_id

class FileInfoResponse(CogniteDataObject):
    'File Info Response Object.\n\n    Attributes:\n        id (int):               ID given by the API to the file.\n        file_name (str):        File name. Max length is 256.\n        directory (str):        Directory containing the file. Max length is 512.\n        source (dict):          Source that this file comes from. Max length is 256.\n        file_type (str):        File type. E.g. pdf, css, spreadsheet, .. Max length is 64.\n        metadata (dict):        Customized data about the file.\n        asset_ids (list[str]):  Names of assets related to this file.\n        uploaded (bool):        Whether or not the file is uploaded.\n        uploaded_at (int):      Epoc thime (ms) when the file was uploaded succesfully.\n    '

    def __init__(self, internal_representation):
        super().__init__(internal_representation)
        item = self.internal_representation['data']['items'][0]
        self.id = item.get('id')
        self.file_name = item.get('fileName')
        self.directory = item.get('directory')
        self.source = item.get('source')
        self.file_type = item.get('fileType')
        self.metadata = item.get('metadata')
        self.asset_ids = item.get('assetIds')
        self.uploaded = item.get('uploaded')
        self.uploaded_at = item.get('uploadedAt')

    def to_json(self):
        return self.internal_representation['data']['items'][0]

    def to_pandas(self):
        file_info = deepcopy(self.to_json())
        if file_info.get('metadata'):
            file_info.update(file_info.pop('metadata'))
        return pd.DataFrame.from_dict(file_info, orient='index')

class FileListResponse(CogniteDataObject):
    'File List Response Object'

    def to_json(self):
        return self.internal_representation['data']['items']

    def to_pandas(self):
        return pd.DataFrame(self.internal_representation['data']['items'])

class EventResponse(CogniteDataObject):
    'Event Response Object.'

    def __init__(self, internal_representation):
        super().__init__(internal_representation)
        item = self.internal_representation['data']['items'][0]
        self.id = item.get('id')
        self.asset_ids = item.get('assetIds')

    def to_json(self):
        return self.internal_representation['data']['items'][0]

    def to_pandas(self):
        event = deepcopy(self.to_json())
        if event.get('metadata'):
            event.update(event.pop('metadata'))
        return pd.DataFrame.from_dict(event, orient='index')

class EventListResponse(CogniteDataObject):
    'Event List Response Object.'

    def __init__(self, internal_representation):
        super().__init__(internal_representation)
        self.counter = 0

    def to_json(self):
        return self.internal_representation['data']['items']

    def to_pandas(self):
        items = deepcopy(self.to_json())
        for d in items:
            if d.get('metadata'):
                d.update(d.pop('metadata'))
        return pd.DataFrame(items)

    def __iter__(self):
        return self

    def __next__(self):
        if (self.counter > (len(self.to_json()) - 1)):
            raise StopIteration
        else:
            self.counter += 1
            return EventResponse({'data': {'items': [self.to_json()[(self.counter - 1)]]}})

class Event(object):
    "Data transfer object for events.\n    Attributes:\n        start_time (int):       Start time of the event in ms since epoch.\n        end_time (int):         End time of the event in ms since epoch.\n        description (str):      Textual description of the event.\n        type (str):             Type of the event, e.g. 'failure'.\n        sub_type (str):          Subtype of the event, e.g. 'electrical'.\n        metadata (dict):        Customizable extra data about the event.\n        asset_ids (list[int]):  List of Asset IDs of related equipments that this event relates to.\n    "

    def __init__(self, start_time=None, end_time=None, description=None, type=None, sub_type=None, metadata=None, asset_ids=None):
        self.startTime = start_time
        self.endTime = end_time
        self.description = description
        self.type = type
        self.subtype = sub_type
        self.metadata = metadata
        self.assetIds = asset_ids




'Cloud Storage Module\n\nThis module mirrors the Cloud Storage API. It allows you to manage files in cloud storage.\n\nhttps://doc.cognitedata.com/0.4/#Cognite-API-Cloud-Storage\n'
import os
import warnings
import requests
import cognite._utils as _utils
import cognite.config as config
from cognite.v04.dto import FileInfoResponse, FileListResponse

def upload_file(file_name, file_path=None, directory=None, source=None, file_type=None, content_type=None, **kwargs):
    'Upload metadata about a file and get an upload link.\n\n    The link will expire after 30 seconds if not resumable. A resumable upload link is default. Such a link is one-time\n    use and expires after one week. For more information, check this link:\n    https://cloud.google.com/storage/docs/json_api/v1/how-tos/resumable-upload. Use PUT request to upload file with the\n    link returned.\n\n    If file_path is specified, the file will be uploaded directly by the SDK.\n\n    Args:\n        file_name (str):      File name. Max length is 256.\n\n        file_path (str, optional):     Path of file to upload, if omitted a upload link will be returned.\n\n        content_type (str, optional):   MIME type of your file. Required if file_path is specified.\n\n        directory (str, optional):      Directory containing the file. Max length is 512.\n\n        source (str, optional):         Source that this file comes from. Max length is 256.\n\n        file_type (str, optional):      File type. E.g. pdf, css, spreadsheet, .. Max length is 64.\n\n    Keyword Args:\n        api_key (str, optional):        Your api-key.\n\n        project (str, optional):        Project name.\n\n        metadata (dict):      Customized data about the file.\n\n        tag_ids (list):       IDs (tagIds) of equipment related to this file.\n\n        resumable (bool):     Whether to generate a resumable URL or not. Default is true.\n\n        overwrite (bool):     Whether to overwrite existing data if duplicate or not. Default is false.\n\n    Returns:\n        dict: A dictionary containing the field fileId and optionally also uploadURL if file_path is omitted.\n    '
    (api_key, project) = config.get_config_variables(kwargs.get('api_key'), kwargs.get('project'))
    url = (config.get_base_url(api_version=0.4) + '/projects/{}/storage/metadata'.format(project))
    headers = {'api-key': api_key, 'content-type': 'application/json', 'accept': 'application/json', 'X-Upload-Content-Type': content_type}
    params = {'resumable': kwargs.get('resumable', True), 'overwrite': kwargs.get('overwrite', False)}
    body = {'fileName': file_name, 'directory': directory, 'source': source, 'fileType': file_type, 'metadata': kwargs.get('metadata', None), 'tagIds': kwargs.get('tagIds', None)}
    res_storage = _utils.post_request(url=url, body=body, headers=headers, params=params, cookies=config.get_cookies())
    result = res_storage.json()['data']
    if file_path:
        if (not content_type):
            warning = 'content_type should be specified when directly uploading the file.'
            warnings.warn(warning)
        headers = {'content-length': str(os.path.getsize(file_path))}
        with open(file_path, 'rb') as file:
            requests.put(result['uploadURL'], data=file, headers=headers)
        result.pop('uploadURL')
    return result

def download_file(id, get_contents=False, **kwargs):
    'Get list of files matching query.\n\n    Args:\n        id (int):                           Path to file to upload, if omitted a upload link will be returned.\n\n        get_contents (bool, optional):      Boolean to determince whether or not to return file contents as string.\n                                            Default is False and download url is returned.\n\n    Keyword Args:\n        api_key (str, optional):            Your api-key.\n\n        project (str, optional):            Project name.\n\n    Returns:\n        str: Download link if get_contents is False else file contents.\n    '
    (api_key, project) = config.get_config_variables(kwargs.get('api_key'), kwargs.get('project'))
    url = (config.get_base_url(api_version=0.4) + '/projects/{}/storage/{}'.format(project, id))
    headers = {'api-key': api_key, 'accept': 'application/json'}
    res = _utils.get_request(url=url, headers=headers, cookies=config.get_cookies())
    if get_contents:
        dl_link = res.json()['data']
        res = requests.get(dl_link)
        return res.content
    return res.json()['data']

def delete_files(file_ids, **kwargs):
    'Delete\n\n    Args:\n        file_ids (list[int]):   List of IDs of files to delete.\n\n    Keyword Args:\n        api_key (str):          Your api key.\n\n        project (str):          Your project.\n\n    Returns:\n        list: List of files deleted and files that failed to delete.\n    '
    (api_key, project) = config.get_config_variables(kwargs.get('api_key'), kwargs.get('project'))
    url = (config.get_base_url(api_version=0.4) + '/projects/{}/storage/delete'.format(project))
    headers = {'api-key': api_key, 'content-type': 'application/json', 'accept': 'application/json'}
    body = {'items': file_ids}
    res = _utils.post_request(url, body=body, headers=headers)
    return res.json()['data']

def list_files(name=None, directory=None, file_type=None, source=None, **kwargs):
    "Get list of files matching query.\n\n    Args:\n        name (str, optional):      List all files with this name.\n\n        directory (str, optional):      Directory to list files from.\n\n        source (str, optional):         List files coming from this source.\n\n        file_type (str, optional):      Type of files to list.\n\n    Keyword Args:\n        api_key (str, optional):        Your api-key.\n\n        project (str, optional):        Project name.\n\n        tag_id (list):                  Returns all files associated with this tagId.\n\n        sort (str):                     Sort descending or ascending. 'ASC' or 'DESC'.\n\n        limit (int):                    Number of results to return.\n\n        autopaging (bool):              Whether or not to automatically page through results. If set to true, limit will be\n                                        disregarded. Defaults to False.\n\n    Returns:\n        v04.dto.FileListResponse: A data object containing the requested files information.\n    "
    (api_key, project) = config.get_config_variables(kwargs.get('api_key'), kwargs.get('project'))
    url = (config.get_base_url(api_version=0.4) + '/projects/{}/storage'.format(project))
    headers = {'api-key': api_key, 'accept': 'application/json'}
    params = {'tagId': kwargs.get('tag_id', None), 'dir': directory, 'name': name, 'type': file_type, 'source': source, 'isUploaded': kwargs.get('is_uploaded', None), 'sort': kwargs.get('sort', None), 'limit': (kwargs.get('limit', 100) if (not kwargs.get('autopaging')) else 10000)}
    file_list = []
    res = _utils.get_request(url=url, headers=headers, params=params, cookies=config.get_cookies())
    file_list.extend(res.json()['data']['items'])
    next_cursor = res.json()['data'].get('nextCursor', None)
    while (next_cursor and kwargs.get('autopaging')):
        params['cursor'] = next_cursor
        res = _utils.get_request(url=url, headers=headers, params=params, cookies=config.get_cookies())
        file_list.extend(res.json()['data']['items'])
        next_cursor = res.json()['data'].get('nextCursor', None)
    return FileListResponse({'data': {'nextCursor': next_cursor, 'previousCursor': res.json()['data'].get('previousCursor'), 'items': file_list}})

def get_file_info(id, **kwargs):
    'Returns information about a file.\n\n    Args:\n        id (int):                   Id of the file.\n\n    Keyword Args:\n        api_key (str, optional):    Your api-key.\n\n        project (str, optional):    Project name.\n\n    Returns:\n        v04.dto.FileInfoResponse: A data object containing the requested file information.\n    '
    (api_key, project) = config.get_config_variables(kwargs.get('api_key'), kwargs.get('project'))
    url = (config.get_base_url(api_version=0.4) + '/projects/{}/storage/{}/info'.format(project, id))
    headers = {'api-key': api_key, 'accept': 'application/json'}
    res = _utils.get_request(url, headers=headers)
    return FileInfoResponse(res.json())



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: api_timeseries_data_v1.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='api_timeseries_data_v1.proto',
  package='api',
  syntax='proto3',
  serialized_pb=_b('\n\x1c\x61pi_timeseries_data_v1.proto\x12\x03\x61pi\"3\n\x0fStringDatapoint\x12\x11\n\ttimestamp\x18\x01 \x01(\x03\x12\r\n\x05value\x18\x02 \x01(\t\"4\n\x10NumericDatapoint\x12\x11\n\ttimestamp\x18\x01 \x01(\x03\x12\r\n\x05value\x18\x02 \x01(\x01\"<\n\x14StringTimeseriesData\x12$\n\x06points\x18\x01 \x03(\x0b\x32\x14.api.StringDatapoint\">\n\x15NumericTimeseriesData\x12%\n\x06points\x18\x01 \x03(\x0b\x32\x15.api.NumericDatapoint\"|\n\x0eTimeseriesData\x12/\n\nstringData\x18\x01 \x01(\x0b\x32\x19.api.StringTimeseriesDataH\x00\x12\x31\n\x0bnumericData\x18\x02 \x01(\x0b\x32\x1a.api.NumericTimeseriesDataH\x00\x42\x06\n\x04\x64\x61ta\"\x8e\x01\n\x11TagTimeseriesData\x12\r\n\x05tagId\x18\x01 \x01(\t\x12/\n\nstringData\x18\x02 \x01(\x0b\x32\x19.api.StringTimeseriesDataH\x00\x12\x31\n\x0bnumericData\x18\x03 \x01(\x0b\x32\x1a.api.NumericTimeseriesDataH\x00\x42\x06\n\x04\x64\x61ta\"K\n\x16MultiTagTimeseriesData\x12\x31\n\x11tagTimeseriesData\x18\x01 \x03(\x0b\x32\x16.api.TagTimeseriesDataB4\n\x17\x63om.cognite.data.api.v1P\x01\xb8\x01\x01\xaa\x02\x13\x43ognite.Data.Api.V1b\x06proto3')
)




_STRINGDATAPOINT = _descriptor.Descriptor(
  name='StringDatapoint',
  full_name='api.StringDatapoint',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='api.StringDatapoint.timestamp', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='api.StringDatapoint.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=37,
  serialized_end=88,
)


_NUMERICDATAPOINT = _descriptor.Descriptor(
  name='NumericDatapoint',
  full_name='api.NumericDatapoint',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='api.NumericDatapoint.timestamp', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='api.NumericDatapoint.value', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=90,
  serialized_end=142,
)


_STRINGTIMESERIESDATA = _descriptor.Descriptor(
  name='StringTimeseriesData',
  full_name='api.StringTimeseriesData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='points', full_name='api.StringTimeseriesData.points', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=144,
  serialized_end=204,
)


_NUMERICTIMESERIESDATA = _descriptor.Descriptor(
  name='NumericTimeseriesData',
  full_name='api.NumericTimeseriesData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='points', full_name='api.NumericTimeseriesData.points', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=206,
  serialized_end=268,
)


_TIMESERIESDATA = _descriptor.Descriptor(
  name='TimeseriesData',
  full_name='api.TimeseriesData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='stringData', full_name='api.TimeseriesData.stringData', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='numericData', full_name='api.TimeseriesData.numericData', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data', full_name='api.TimeseriesData.data',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=270,
  serialized_end=394,
)


_TAGTIMESERIESDATA = _descriptor.Descriptor(
  name='TagTimeseriesData',
  full_name='api.TagTimeseriesData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tagId', full_name='api.TagTimeseriesData.tagId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stringData', full_name='api.TagTimeseriesData.stringData', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='numericData', full_name='api.TagTimeseriesData.numericData', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data', full_name='api.TagTimeseriesData.data',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=397,
  serialized_end=539,
)


_MULTITAGTIMESERIESDATA = _descriptor.Descriptor(
  name='MultiTagTimeseriesData',
  full_name='api.MultiTagTimeseriesData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tagTimeseriesData', full_name='api.MultiTagTimeseriesData.tagTimeseriesData', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=541,
  serialized_end=616,
)

_STRINGTIMESERIESDATA.fields_by_name['points'].message_type = _STRINGDATAPOINT
_NUMERICTIMESERIESDATA.fields_by_name['points'].message_type = _NUMERICDATAPOINT
_TIMESERIESDATA.fields_by_name['stringData'].message_type = _STRINGTIMESERIESDATA
_TIMESERIESDATA.fields_by_name['numericData'].message_type = _NUMERICTIMESERIESDATA
_TIMESERIESDATA.oneofs_by_name['data'].fields.append(
  _TIMESERIESDATA.fields_by_name['stringData'])
_TIMESERIESDATA.fields_by_name['stringData'].containing_oneof = _TIMESERIESDATA.oneofs_by_name['data']
_TIMESERIESDATA.oneofs_by_name['data'].fields.append(
  _TIMESERIESDATA.fields_by_name['numericData'])
_TIMESERIESDATA.fields_by_name['numericData'].containing_oneof = _TIMESERIESDATA.oneofs_by_name['data']
_TAGTIMESERIESDATA.fields_by_name['stringData'].message_type = _STRINGTIMESERIESDATA
_TAGTIMESERIESDATA.fields_by_name['numericData'].message_type = _NUMERICTIMESERIESDATA
_TAGTIMESERIESDATA.oneofs_by_name['data'].fields.append(
  _TAGTIMESERIESDATA.fields_by_name['stringData'])
_TAGTIMESERIESDATA.fields_by_name['stringData'].containing_oneof = _TAGTIMESERIESDATA.oneofs_by_name['data']
_TAGTIMESERIESDATA.oneofs_by_name['data'].fields.append(
  _TAGTIMESERIESDATA.fields_by_name['numericData'])
_TAGTIMESERIESDATA.fields_by_name['numericData'].containing_oneof = _TAGTIMESERIESDATA.oneofs_by_name['data']
_MULTITAGTIMESERIESDATA.fields_by_name['tagTimeseriesData'].message_type = _TAGTIMESERIESDATA
DESCRIPTOR.message_types_by_name['StringDatapoint'] = _STRINGDATAPOINT
DESCRIPTOR.message_types_by_name['NumericDatapoint'] = _NUMERICDATAPOINT
DESCRIPTOR.message_types_by_name['StringTimeseriesData'] = _STRINGTIMESERIESDATA
DESCRIPTOR.message_types_by_name['NumericTimeseriesData'] = _NUMERICTIMESERIESDATA
DESCRIPTOR.message_types_by_name['TimeseriesData'] = _TIMESERIESDATA
DESCRIPTOR.message_types_by_name['TagTimeseriesData'] = _TAGTIMESERIESDATA
DESCRIPTOR.message_types_by_name['MultiTagTimeseriesData'] = _MULTITAGTIMESERIESDATA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

StringDatapoint = _reflection.GeneratedProtocolMessageType('StringDatapoint', (_message.Message,), dict(
  DESCRIPTOR = _STRINGDATAPOINT,
  __module__ = 'api_timeseries_data_v1_pb2'
  # @@protoc_insertion_point(class_scope:api.StringDatapoint)
  ))
_sym_db.RegisterMessage(StringDatapoint)

NumericDatapoint = _reflection.GeneratedProtocolMessageType('NumericDatapoint', (_message.Message,), dict(
  DESCRIPTOR = _NUMERICDATAPOINT,
  __module__ = 'api_timeseries_data_v1_pb2'
  # @@protoc_insertion_point(class_scope:api.NumericDatapoint)
  ))
_sym_db.RegisterMessage(NumericDatapoint)

StringTimeseriesData = _reflection.GeneratedProtocolMessageType('StringTimeseriesData', (_message.Message,), dict(
  DESCRIPTOR = _STRINGTIMESERIESDATA,
  __module__ = 'api_timeseries_data_v1_pb2'
  # @@protoc_insertion_point(class_scope:api.StringTimeseriesData)
  ))
_sym_db.RegisterMessage(StringTimeseriesData)

NumericTimeseriesData = _reflection.GeneratedProtocolMessageType('NumericTimeseriesData', (_message.Message,), dict(
  DESCRIPTOR = _NUMERICTIMESERIESDATA,
  __module__ = 'api_timeseries_data_v1_pb2'
  # @@protoc_insertion_point(class_scope:api.NumericTimeseriesData)
  ))
_sym_db.RegisterMessage(NumericTimeseriesData)

TimeseriesData = _reflection.GeneratedProtocolMessageType('TimeseriesData', (_message.Message,), dict(
  DESCRIPTOR = _TIMESERIESDATA,
  __module__ = 'api_timeseries_data_v1_pb2'
  # @@protoc_insertion_point(class_scope:api.TimeseriesData)
  ))
_sym_db.RegisterMessage(TimeseriesData)

TagTimeseriesData = _reflection.GeneratedProtocolMessageType('TagTimeseriesData', (_message.Message,), dict(
  DESCRIPTOR = _TAGTIMESERIESDATA,
  __module__ = 'api_timeseries_data_v1_pb2'
  # @@protoc_insertion_point(class_scope:api.TagTimeseriesData)
  ))
_sym_db.RegisterMessage(TagTimeseriesData)

MultiTagTimeseriesData = _reflection.GeneratedProtocolMessageType('MultiTagTimeseriesData', (_message.Message,), dict(
  DESCRIPTOR = _MULTITAGTIMESERIESDATA,
  __module__ = 'api_timeseries_data_v1_pb2'
  # @@protoc_insertion_point(class_scope:api.MultiTagTimeseriesData)
  ))
_sym_db.RegisterMessage(MultiTagTimeseriesData)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\027com.cognite.data.api.v1P\001\270\001\001\252\002\023Cognite.Data.Api.V1'))
# @@protoc_insertion_point(module_scope)



from datetime import datetime
from random import randint

import numpy as np
import pandas as pd
import pytest

from cognite.v05 import depthseries, timeseries, dto

DS_NAME = None


@pytest.fixture(autouse=True, scope='class')
def ts_name():
    global DS_NAME
    DS_NAME = 'test_ds_{}'.format(randint(1, 2 ** 53 - 1))


class TestDepthseries:
    @pytest.fixture(scope='class', params=[True, False])
    def get_depthseries_response_obj(self ):
        yield depthseries.get_depthseries(prefix=DS_NAME, limit=1)

    def test_post_depthseries(self):
        tso = dto.TimeSeries(DS_NAME)
        res = depthseries.post_depth_series([tso])
        assert res == {}

    def test_post_datapoints(self):
        dps = [dto.DatapointDepth(i, i * 100) for i in range(10)]
        res = depthseries.post_datapoints(DS_NAME, depthdatapoints=dps)
        assert res == {}

    def test_get_latest(self):
        from cognite.v05.dto import LatestDatapointResponse
        response = depthseries.get_latest(DS_NAME)
        assert isinstance(response, LatestDatapointResponse)
        assert isinstance(response.to_ndarray(), np.ndarray)
        assert isinstance(response.to_pandas(), pd.DataFrame)
        assert isinstance(response.to_json(), dict)

    def test_update_timeseries(self):
        tso = dto.TimeSeries(DS_NAME, unit='celsius')
        res = depthseries.update_depth_series([tso])
        assert res == {}

    def test_depthseries_unit_correct(self, get_depthseries_response_obj):
        assert get_depthseries_response_obj.to_json()[0]['unit'] == 'celsius'

    def test_get_depthseries_output_format(self, get_depthseries_response_obj):
        print(get_depthseries_response_obj.to_pandas())
        from cognite.v05.dto import TimeSeriesResponse
        assert isinstance(get_depthseries_response_obj, TimeSeriesResponse)
        assert isinstance(get_depthseries_response_obj.to_ndarray(), np.ndarray)
        assert isinstance(get_depthseries_response_obj.to_pandas(), pd.DataFrame)
        assert isinstance(get_depthseries_response_obj.to_json()[0], dict)

    def test_get_depthseries_no_results(self):
        result = depthseries.get_depthseries(prefix='not_a_depthseries_prefix')
        assert result.to_pandas().empty
        assert len(result.to_json()) == 0

    def test_delete_depthseries(self):
        res = depthseries.delete_depth_series(DS_NAME)
        assert res == {}

    def test_reset_depthseries(self):
        res = depthseries.reset_depth_series(DS_NAME)
        assert res == {}




'Events Module\n\nThis module mirrors the Events API. It allows you to get, post, update, and delete events.\n\nhttps://doc.cognitedata.com/0.5/#Cognite-API-Events\n'
from cognite import _utils, config, _constants
from cognite.v05.dto import EventResponse, EventListResponse

def get_event(event_id, **kwargs):
    'Returns a EventResponse containing an event matching the id.\n\n    Args:\n        event_id (int):         The event id.\n\n    Keyword Arguments:\n        api_key (str):          Your api-key.\n\n        project (str):          Project name.\n\n    Returns:\n        v05.dto.EventResponse: A data object containing the requested event.\n    '
    (api_key, project) = config.get_config_variables(kwargs.get('api_key'), kwargs.get('project'))
    url = (config.get_base_url(api_version=0.5) + '/projects/{}/events/{}'.format(project, event_id))
    headers = {'api-key': api_key, 'content-type': 'application/json', 'accept': 'application/json'}
    res = _utils.get_request(url, headers=headers, cookies=config.get_cookies())
    return EventResponse(res.json())

def get_events(type=None, sub_type=None, asset_id=None, **kwargs):
    "Returns an EventListReponse object containing events matching the query.\n\n    Args:\n        type (str):             Type (class) of event, e.g. 'failure'.\n        sub_type (str):         Sub-type of event, e.g. 'electrical'.\n        asset_id (str):         Return events associated with this assetId.\n    Keyword Arguments:\n        sort (str):             Sort descending or ascending. Default 'ASC'.\n        cursor (str):           Cursor to use for paging through results.\n        limit (int):            Return up to this many results. Maximum is 10000. Default is 25.\n        has_description (bool): Return only events that have a textual description. Default null. False gives only\n                                those without description.\n        min_start_time (string): Only return events from after this time.\n        max_start_time (string): Only return events form before this time.\n        api_key (str):          Your api-key.\n        project (str):          Project name.\n        autopaging (bool):      Whether or not to automatically page through results. If set to true, limit will be\n                                disregarded. Defaults to False.\n\n    Returns:\n        v05.dto.EventListResponse: A data object containing the requested event.\n    "
    (api_key, project) = config.get_config_variables(kwargs.get('api_key'), kwargs.get('project'))
    url = (config.get_base_url(api_version=0.5) + '/projects/{}/events'.format(project))
    headers = {'api-key': api_key, 'content-type': 'application/json', 'accept': 'application/json'}
    params = {'type': type, 'subtype': sub_type, 'assetId': asset_id, 'sort': kwargs.get('sort'), 'cursor': kwargs.get('cursor'), 'limit': (kwargs.get('limit', 25) if (not kwargs.get('autopaging')) else _constants.LIMIT_AGG), 'hasDescription': kwargs.get('has_description'), 'minStartTime': kwargs.get('min_start_time'), 'maxStartTime': kwargs.get('max_start_time')}
    res = _utils.get_request(url, headers=headers, params=params, cookies=config.get_cookies())
    events = []
    events.extend(res.json()['data']['items'])
    next_cursor = res.json()['data'].get('nextCursor')
    while (next_cursor and kwargs.get('autopaging')):
        params['cursor'] = next_cursor
        res = _utils.get_request(url=url, headers=headers, params=params, cookies=config.get_cookies())
        events.extend(res.json()['data']['items'])
        next_cursor = res.json()['data'].get('nextCursor')
    return EventListResponse({'data': {'nextCursor': next_cursor, 'previousCursor': res.json()['data'].get('previousCursor'), 'items': events}})

def post_events(events, **kwargs):
    'Adds a list of events and returns an EventListResponse object containing created events.\n\n    Args:\n        events (List[v05.dto.Event]):    List of events to create.\n\n    Keyword Args:\n        api_key (str):          Your api-key.\n        project (str):          Project name.\n\n    Returns:\n        v05.dto.EventListResponse\n    '
    (api_key, project) = config.get_config_variables(kwargs.get('api_key'), kwargs.get('project'))
    url = (config.get_base_url(api_version=0.5) + '/projects/{}/events'.format(project))
    headers = {'api-key': api_key, 'content-type': 'application/json', 'accept': 'application/json'}
    body = {'items': [event.__dict__ for event in events]}
    res = _utils.post_request(url, body=body, headers=headers)
    return EventListResponse(res.json())

def delete_events(event_ids, **kwargs):
    'Deletes a list of events.\n\n    Args:\n        event_ids (List[int]):    List of ids of events to delete.\n\n    Keyword Args:\n        api_key (str):          Your api-key.\n        project (str):          Project name.\n\n    Returns:\n        An empty response.\n    '
    (api_key, project) = config.get_config_variables(kwargs.get('api_key'), kwargs.get('project'))
    url = (config.get_base_url(api_version=0.5) + '/projects/{}/events/delete'.format(project))
    headers = {'api-key': api_key, 'content-type': 'application/json', 'accept': 'application/json'}
    body = {'items': event_ids}
    res = _utils.post_request(url, body=body, headers=headers)
    return res.json()



from datetime import datetime
from random import randint

import numpy as np
import pandas as pd
import pytest

from cognite.v05 import timeseries, dto

TS_NAME = None

dps_params = [
    {'start': 1522188000000, 'end': 1522620000000},
    {'start': datetime(2018, 4, 1), 'end': datetime(2018, 4, 2)},
    {'start': datetime(2018, 4, 1), 'end': datetime(2018, 4, 2), 'protobuf': True}]


@pytest.fixture(autouse=True, scope='class')
def ts_name():
    global TS_NAME
    TS_NAME = 'test_ts_{}'.format(randint(1, 2 ** 53 - 1))


class TestTimeseries:
    @pytest.fixture(scope='class', params=[True, False])
    def get_timeseries_response_obj(self, request):
        yield timeseries.get_timeseries(prefix=TS_NAME, limit=1, include_metadata=request.param)

    def test_post_timeseries(self):
        tso = dto.TimeSeries(TS_NAME)
        res = timeseries.post_time_series([tso])
        assert res == {}

    def test_update_timeseries(self):
        tso = dto.TimeSeries(TS_NAME, unit='celsius')
        res = timeseries.update_time_series([tso])
        assert res == {}

    def test_timeseries_unit_correct(self, get_timeseries_response_obj):
        assert get_timeseries_response_obj.to_json()[0]['unit'] == 'celsius'

    def test_get_timeseries_output_format(self, get_timeseries_response_obj):
        print(get_timeseries_response_obj.to_pandas())
        from cognite.v05.dto import TimeSeriesResponse
        assert isinstance(get_timeseries_response_obj, TimeSeriesResponse)
        assert isinstance(get_timeseries_response_obj.to_ndarray(), np.ndarray)
        assert isinstance(get_timeseries_response_obj.to_pandas(), pd.DataFrame)
        assert isinstance(get_timeseries_response_obj.to_json()[0], dict)

    def test_get_timeseries_no_results(self):
        result = timeseries.get_timeseries(prefix='not_a_timeseries_prefix')
        assert result.to_pandas().empty
        assert len(result.to_json()) == 0

    def test_delete_timeseries(self):
        res = timeseries.delete_time_series(TS_NAME)
        assert res == {}


@pytest.fixture(scope='class')
def datapoints_fixture():
    tso = dto.TimeSeries(TS_NAME)
    timeseries.post_time_series([tso])
    yield
    timeseries.delete_time_series(TS_NAME)


@pytest.mark.usefixtures('datapoints_fixture')
class TestDatapoints:
    @pytest.fixture(scope='class', params=dps_params)
    def get_dps_response_obj(self, request):
        yield timeseries.get_datapoints(name='constant', start=request.param['start'], end=request.param['end'],
                                        protobuf=request.param.get('protobuf', False))

    def test_post_datapoints(self):
        dps = [dto.Datapoint(i, i * 100) for i in range(10)]
        res = timeseries.post_datapoints(TS_NAME, datapoints=dps)
        assert res == {}

    def test_get_datapoints(self, get_dps_response_obj):
        from cognite.v05.dto import DatapointsResponse
        assert isinstance(get_dps_response_obj, DatapointsResponse)

    def test_get_dps_output_formats(self, get_dps_response_obj):
        assert isinstance(get_dps_response_obj.to_ndarray(), np.ndarray)
        assert isinstance(get_dps_response_obj.to_pandas(), pd.DataFrame)
        assert isinstance(get_dps_response_obj.to_json(), dict)

    def test_get_dps_correctly_spaced(self, get_dps_response_obj):
        timestamps = get_dps_response_obj.to_pandas().timestamp.values
        deltas = np.diff(timestamps, 1)
        assert (deltas != 0).all()
        assert (deltas % 10000 == 0).all()


class TestLatest:
    def test_get_latest(self):
        from cognite.v05.dto import LatestDatapointResponse
        response = timeseries.get_latest('constant')
        assert isinstance(response, LatestDatapointResponse)
        assert isinstance(response.to_ndarray(), np.ndarray)
        assert isinstance(response.to_pandas(), pd.DataFrame)
        assert isinstance(response.to_json(), dict)


class TestDatapointsFrame:
    @pytest.fixture(scope='class', params=dps_params[:2])
    def get_datapoints_frame_response_obj(self, request):
        yield timeseries.get_datapoints_frame(time_series=['constant'], start=request.param['start'],
                                              end=request.param['end'],
                                              aggregates=['avg'], granularity='1m')

    def test_get_dps_frame_output_format(self, get_datapoints_frame_response_obj):
        assert isinstance(get_datapoints_frame_response_obj, pd.DataFrame)

    def test_get_dps_frame_correctly_spaced(self, get_datapoints_frame_response_obj):
        timestamps = get_datapoints_frame_response_obj.timestamp.values
        deltas = np.diff(timestamps, 1)
        assert (deltas != 0).all()
        assert (deltas % 60000 == 0).all()


class TestMultiTimeseriesDatapoints:
    @pytest.fixture(scope='class', params=dps_params[:2])
    def get_multi_time_series_dps_response_obj(self, request):
        from cognite.v05.dto import DatapointsQuery
        dq1 = DatapointsQuery('constant')
        dq2 = DatapointsQuery('sinus', aggregates=['avg'], granularity='30s')
        yield list(
            timeseries.get_multi_time_series_datapoints(datapoints_queries=[dq1, dq2], start=request.param['start'],
                                                        end=request.param['end'], aggregates=['avg'],
                                                        granularity='60s'))

    @staticmethod
    def test_post_multitag_datapoints():
        from cognite.v05 import dto
        from cognite.v05.dto import TimeseriesWithDatapoints
        from unittest import mock
        import cognite._utils as utils

        timeseries_with_too_many_datapoints: TimeseriesWithDatapoints = TimeseriesWithDatapoints(
            name="test",
            datapoints=[dto.Datapoint(x, x) for x in range(100001)]
        )
        timeseries_with_99999_datapoints: TimeseriesWithDatapoints = TimeseriesWithDatapoints(
            name="test",
            datapoints=[dto.Datapoint(x, x) for x in range(99999)]
        )

        with mock.patch.object(utils, 'post_request') as post_request_mock:
            post_request_mock: mock.MagicMock = post_request_mock

            timeseries.post_multi_tag_datapoints([timeseries_with_too_many_datapoints])
            assert post_request_mock.call_count == 2

        with mock.patch.object(utils, 'post_request') as post_request_mock:
            post_request_mock: mock.MagicMock = post_request_mock

            timeseries.post_multi_tag_datapoints([
                timeseries_with_99999_datapoints,
                timeseries_with_too_many_datapoints
            ])
            assert post_request_mock.call_count == 2

    def test_get_multi_time_series_dps_output_format(self, get_multi_time_series_dps_response_obj):
        from cognite.v05.dto import DatapointsResponse
        assert isinstance(get_multi_time_series_dps_response_obj, list)
        for dpr in get_multi_time_series_dps_response_obj:
            assert isinstance(dpr, DatapointsResponse)

    def test_get_multi_time_series_dps_response_length(self, get_multi_time_series_dps_response_obj):
        assert len(list(get_multi_time_series_dps_response_obj)) == 2

    def test_get_multi_timeseries_dps_correctly_spaced(self, get_multi_time_series_dps_response_obj):
        m = list(get_multi_time_series_dps_response_obj)
        timestamps = m[0].to_pandas().timestamp.values
        deltas = np.diff(timestamps, 1)
        assert (deltas != 0).all()
        assert (deltas % 60000 == 0).all()
        timestamps = m[1].to_pandas().timestamp.values
        deltas = np.diff(timestamps, 1)
        assert (deltas != 0).all()
        assert (deltas % 30000 == 0).all()


def test_split_TimeseriesWithDatapoints_if_over_limit():
    from cognite.v05.dto import TimeseriesWithDatapoints
    from cognite.v05.dto import Datapoint
    from cognite.v05.timeseries import _split_TimeseriesWithDatapoints_if_over_limit
    from typing import List

    timeseries_with_datapoints_over_limit: TimeseriesWithDatapoints = TimeseriesWithDatapoints(
        name="test",
        datapoints=[Datapoint(x, x) for x in range(1000)]
    )

    result: List[TimeseriesWithDatapoints] = _split_TimeseriesWithDatapoints_if_over_limit(
        timeseries_with_datapoints_over_limit,
        100
    )

    assert isinstance(result[0], TimeseriesWithDatapoints)
    assert len(result) == 10

    result = _split_TimeseriesWithDatapoints_if_over_limit(
        timeseries_with_datapoints_over_limit,
        1000
    )

    assert isinstance(result[0], TimeseriesWithDatapoints)
    assert len(result) == 1



from datetime import datetime
from random import randint

import numpy as np
import pandas as pd
import pytest

from cognite.v04 import timeseries

TS_NAME = None

dps_params = [
    {'start': 1522188000000, 'end': 1522620000000},
    {'start': datetime(2018, 4, 1), 'end': datetime(2018, 4, 2)},
    {'start': datetime(2018, 4, 1), 'end': datetime(2018, 4, 2), 'protobuf': True}]


@pytest.fixture(scope='module', params=dps_params)
def get_dps_response_obj(request):
    yield timeseries.get_datapoints(tag_id='constant', start=request.param['start'], end=request.param['end'],
                                    protobuf=request.param.get('protobuf', False))


def test_get_datapoints(get_dps_response_obj):
    from cognite.v04.dto import DatapointsResponse
    assert isinstance(get_dps_response_obj, DatapointsResponse)


def test_get_dps_output_formats(get_dps_response_obj):
    assert isinstance(get_dps_response_obj.to_ndarray(), np.ndarray)
    assert isinstance(get_dps_response_obj.to_pandas(), pd.DataFrame)
    assert isinstance(get_dps_response_obj.to_json(), dict)


def test_get_dps_correctly_spaced(get_dps_response_obj):
    timestamps = get_dps_response_obj.to_pandas().timestamp.values
    deltas = np.diff(timestamps, 1)
    assert (deltas != 0).all()
    assert (deltas % 10000 == 0).all()


def test_get_latest():
    from cognite.v04.dto import LatestDatapointResponse
    response = timeseries.get_latest('constant')
    assert isinstance(response, LatestDatapointResponse)
    assert isinstance(response.to_ndarray(), np.ndarray)
    assert isinstance(response.to_pandas(), pd.DataFrame)
    assert isinstance(response.to_json(), dict)


@pytest.fixture(scope='module', params=dps_params[:2])
def get_datapoints_frame_response_obj(request):
    yield timeseries.get_datapoints_frame(tag_ids=['constant'], start=request.param['start'], end=request.param['end'],
                                          aggregates=['avg'], granularity='1m')


def test_get_dps_frame_output_format(get_datapoints_frame_response_obj):
    assert isinstance(get_datapoints_frame_response_obj, pd.DataFrame)


def test_get_dps_frame_correctly_spaced(get_datapoints_frame_response_obj):
    timestamps = get_datapoints_frame_response_obj.timestamp.values
    deltas = np.diff(timestamps, 1)
    assert (deltas != 0).all()
    assert (deltas % 60000 == 0).all()


@pytest.fixture(scope='module', params=dps_params[:2])
def get_multitag_dps_response_obj(request):
    from cognite.v04.dto import DatapointsQuery
    dq1 = DatapointsQuery('constant')
    dq2 = DatapointsQuery('sinus', aggregates=['avg'], granularity='30s')
    yield list(timeseries.get_multi_tag_datapoints(datapoints_queries=[dq1, dq2], start=request.param['start'],
                                                   end=request.param['end'], aggregates=['avg'], granularity='60s'))


def test_get_multitag_dps_output_format(get_multitag_dps_response_obj):
    from cognite.v04.dto import DatapointsResponse
    assert isinstance(get_multitag_dps_response_obj, list)
    for dpr in get_multitag_dps_response_obj:
        assert isinstance(dpr, DatapointsResponse)


def test_get_multitag_dps_response_length(get_multitag_dps_response_obj):
    assert len(list(get_multitag_dps_response_obj)) == 2


def test_get_multitag_dps_correctly_spaced(get_multitag_dps_response_obj):
    m = list(get_multitag_dps_response_obj)
    timestamps = m[0].to_pandas().timestamp.values
    deltas = np.diff(timestamps, 1)
    assert (deltas != 0).all()
    assert (deltas % 60000 == 0).all()
    timestamps = m[1].to_pandas().timestamp.values
    deltas = np.diff(timestamps, 1)
    assert (deltas != 0).all()
    assert (deltas % 30000 == 0).all()


@pytest.fixture(autouse=True, scope='class')
def ts_name():
    global TS_NAME
    TS_NAME = 'test_ts_{}'.format(randint(1, 2 ** 53 - 1))


@pytest.fixture(scope='class')
def timeseries_fixture():
    from cognite.v05 import timeseries
    yield
    timeseries.delete_time_series(TS_NAME)


@pytest.mark.usefixtures('timeseries_fixture')
class TestTimeseries:
    @pytest.fixture(scope='class', params=[True, False])
    def get_timeseries_response_obj(self, request):
        yield timeseries.get_timeseries(prefix=TS_NAME, limit=1, include_metadata=request.param)

    def test_post_timeseries(self):
        from cognite.v04 import dto
        tso = dto.TimeSeries(TS_NAME)
        res = timeseries.post_time_series([tso])
        assert res == {}

    @staticmethod
    def test_post_multitag_datapoints():
        from cognite.v04 import dto
        from cognite.v04.dto import TimeseriesWithDatapoints
        from unittest import mock
        import cognite._utils as utils

        timeseries_with_too_many_datapoints: TimeseriesWithDatapoints = TimeseriesWithDatapoints(
            tagId="test",
            datapoints=[dto.Datapoint(x, x) for x in range(100001)]
        )
        timeseries_with_99999_datapoints: TimeseriesWithDatapoints = TimeseriesWithDatapoints(
            tagId="test",
            datapoints=[dto.Datapoint(x, x) for x in range(99999)]
        )

        with mock.patch.object(utils, 'post_request') as post_request_mock:
            post_request_mock: mock.MagicMock = post_request_mock

            timeseries.post_multi_tag_datapoints([timeseries_with_too_many_datapoints])
            assert post_request_mock.call_count == 2

        with mock.patch.object(utils, 'post_request') as post_request_mock:
            post_request_mock: mock.MagicMock = post_request_mock

            timeseries.post_multi_tag_datapoints([
                timeseries_with_99999_datapoints,
                timeseries_with_too_many_datapoints
            ])
            assert post_request_mock.call_count == 2

    def test_update_timeseries(self):
        from cognite.v04 import dto
        tso = dto.TimeSeries(TS_NAME, unit='celsius')
        res = timeseries.update_time_series([tso])
        assert res == {}

    def test_timeseries_unit_correct(self, get_timeseries_response_obj):
        assert get_timeseries_response_obj.to_json()[0]['unit'] == 'celsius'

    def test_get_timeseries_output_format(self, get_timeseries_response_obj):
        from cognite.v04.dto import TimeSeriesResponse
        assert isinstance(get_timeseries_response_obj, TimeSeriesResponse)
        assert isinstance(get_timeseries_response_obj.to_ndarray(), np.ndarray)
        assert isinstance(get_timeseries_response_obj.to_pandas(), pd.DataFrame)
        assert isinstance(get_timeseries_response_obj.to_json()[0], dict)

    def test_get_timeseries_no_results(self):
        result = timeseries.get_timeseries(prefix='not_a_timeseries_prefix')
        assert result.to_pandas().empty
        assert len(result.to_json()) == 0


def test_split_TimeseriesWithDatapoints_if_over_limit():
    from cognite.v04.dto import TimeseriesWithDatapoints
    from cognite.v04.dto import Datapoint
    from cognite.v04.timeseries import _split_TimeseriesWithDatapoints_if_over_limit
    from typing import List

    timeseries_with_datapoints_over_limit: TimeseriesWithDatapoints = TimeseriesWithDatapoints(
        tagId="test",
        datapoints=[Datapoint(x, x) for x in range(1000)]
    )

    result: List[TimeseriesWithDatapoints] = _split_TimeseriesWithDatapoints_if_over_limit(
        timeseries_with_datapoints_over_limit,
        100
    )

    assert isinstance(result[0], TimeseriesWithDatapoints)
    assert len(result) == 10

    result = _split_TimeseriesWithDatapoints_if_over_limit(
        timeseries_with_datapoints_over_limit,
        1000
    )

    assert isinstance(result[0], TimeseriesWithDatapoints)
    assert len(result) == 1




'Project Configuration Module.\n\nThis module allows you to set an api-key and a project for your python project.\n'
from cognite._constants import BASE_URL, RETRY_LIMIT
_CONFIG_API_KEY = ''
_CONFIG_PROJECT = ''
_CONFIG_COOKIES = {}
_CONFIG_BASE_URL = None
_CONFIG_RETRIES = None

def configure_session(api_key='', project='', cookies=None):
    'Sets session variables.\n\n    Args:\n        api_key (str):  Api-key for current project.\n\n        cookies (dict): Cookies to pass with requests.\n\n        project (str):  Project name for current project.\n    '
    global _CONFIG_API_KEY, _CONFIG_COOKIES, _CONFIG_PROJECT
    _CONFIG_API_KEY = api_key
    _CONFIG_PROJECT = project
    _CONFIG_COOKIES = ({} if (cookies is None) else cookies)

def get_config_variables(api_key, project):
    'Returns current project config variables unless other is specified.\n\n    Args:\n        api_key (str):  Other specified api-key.\n\n        project (str):  Other specified project name.\n\n    Returns:\n        tuple: api-key and project name belonging to current project unless other is specified.\n    '
    if (api_key is None):
        api_key = _CONFIG_API_KEY
    if (project is None):
        project = _CONFIG_PROJECT
    return (api_key, project)

def get_cookies():
    'Returns cookies set for the current session.\n\n    Returns:\n        dict: Cookies for current session.\n    '
    return _CONFIG_COOKIES

def set_base_url(url=None):
    'Sets the base url for requests made from the SDK.\n\n    Args:\n        url (str):  URL to set. Set this to None to use default url.\n    '
    global _CONFIG_BASE_URL
    _CONFIG_BASE_URL = url

def get_base_url(api_version=None):
    'Returns the current base url for requests made from the SDK.\n    Args:\n        api_version (float): Version of API to use for base_url\n    Returns:\n        str: current base url.\n    '
    if (not api_version):
        api_version = '<version>'
    return ((_CONFIG_BASE_URL or BASE_URL) + str(api_version))

def set_number_of_retries(retries=None):
    'Sets the number of retries attempted for requests made from the SDK.\n\n    Args:\n        retries (int):  Number of retries to attempt. Set this to None to use default num of retries.\n    '
    global _CONFIG_RETRIES
    _CONFIG_RETRIES = retries

def get_number_of_retries():
    'Returns the current number of retries attempted for requests made from the SDK.\n\n    Returns:\n        int: current number of retries attempted for requests.\n    '
    return (RETRY_LIMIT if (_CONFIG_RETRIES is None) else _CONFIG_RETRIES)




'Constants\n\nThis module contains constants used in the Cognite Python SDK.\n\nThis module is protected and should not used by end-users.\n\nAttributes:\n    BASE_URL (str):    Base url for Cognite API. Should have correct version number.\n    LIMIT (int):       Limit on how many datapoints should be returned from the API when fetching data using the\n                        timeseries module.\n    RETRY_LIMIT (int): Number of retries to perform if a request to the API should fail.\n'
BASE_URL = 'https://api.cognitedata.com/api/'
LIMIT = 100000
LIMIT_AGG = 10000
RETRY_LIMIT = 1



from copy import deepcopy

import pytest

from cognite.v05.dto import EventResponse, EventListResponse, TimeSeriesResponse, FileInfoResponse


@pytest.fixture(scope='module', params=['ts', 'file', 'event', 'eventlist'])
def get_response_obj(request):
    TS_INTERNAL_REPR = {'data': {'items': [{'name': '0', 'metadata': {'md1': 'val1'}}]}}
    EVENT_LIST_INTERNAL_REPR = {'data': {'items': [{'id': 0, 'metadata': {'md1': 'val1'}}]}}
    EVENT_INTERNAL_REPR = {'data': {'items': [{'id': 0, 'metadata': {'md1': 'val1'}, 'assetIds': []}]}}
    FILE_INFO_INTERNAL_REPR = {'data': {'items': [{'id': 0, 'metadata': {'md1': 'val1'}}]}}

    response = None
    if request.param == 'ts':
        response = TimeSeriesResponse(TS_INTERNAL_REPR)
    elif request.param == 'file':
        response = FileInfoResponse(FILE_INFO_INTERNAL_REPR)
    elif request.param == 'eventlist':
        response = EventListResponse(EVENT_LIST_INTERNAL_REPR)
    elif request.param == 'event':
        response = EventResponse(EVENT_INTERNAL_REPR)

    yield response


class TestDTOs:
    def test_internal_representation_not_mutated(self, get_response_obj):
        repr = deepcopy(get_response_obj.internal_representation)
        get_response_obj.to_ndarray()
        get_response_obj.to_pandas()
        get_response_obj.to_json()
        assert repr == get_response_obj.internal_representation



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: api_timeseries_data_v2.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='api_timeseries_data_v2.proto',
  package='api.v2',
  syntax='proto3',
  serialized_pb=_b('\n\x1c\x61pi_timeseries_data_v2.proto\x12\x06\x61pi.v2\"3\n\x0fStringDatapoint\x12\x11\n\ttimestamp\x18\x01 \x01(\x03\x12\r\n\x05value\x18\x02 \x01(\t\"4\n\x10NumericDatapoint\x12\x11\n\ttimestamp\x18\x01 \x01(\x03\x12\r\n\x05value\x18\x02 \x01(\x01\"?\n\x14StringTimeseriesData\x12\'\n\x06points\x18\x01 \x03(\x0b\x32\x17.api.v2.StringDatapoint\"A\n\x15NumericTimeseriesData\x12(\n\x06points\x18\x01 \x03(\x0b\x32\x18.api.v2.NumericDatapoint\"\x82\x01\n\x0eTimeseriesData\x12\x32\n\nstringData\x18\x01 \x01(\x0b\x32\x1c.api.v2.StringTimeseriesDataH\x00\x12\x34\n\x0bnumericData\x18\x02 \x01(\x0b\x32\x1d.api.v2.NumericTimeseriesDataH\x00\x42\x06\n\x04\x64\x61ta\"\x95\x01\n\x13NamedTimeseriesData\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x32\n\nstringData\x18\x02 \x01(\x0b\x32\x1c.api.v2.StringTimeseriesDataH\x00\x12\x34\n\x0bnumericData\x18\x03 \x01(\x0b\x32\x1d.api.v2.NumericTimeseriesDataH\x00\x42\x06\n\x04\x64\x61ta\"T\n\x18MultiNamedTimeseriesData\x12\x38\n\x13namedTimeseriesData\x18\x01 \x03(\x0b\x32\x1b.api.v2.NamedTimeseriesDataB1\n\x17\x63om.cognite.data.api.v2P\x01\xaa\x02\x13\x43ognite.Data.Api.V2b\x06proto3')
)




_STRINGDATAPOINT = _descriptor.Descriptor(
  name='StringDatapoint',
  full_name='api.v2.StringDatapoint',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='api.v2.StringDatapoint.timestamp', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='api.v2.StringDatapoint.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=40,
  serialized_end=91,
)


_NUMERICDATAPOINT = _descriptor.Descriptor(
  name='NumericDatapoint',
  full_name='api.v2.NumericDatapoint',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='api.v2.NumericDatapoint.timestamp', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='api.v2.NumericDatapoint.value', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=93,
  serialized_end=145,
)


_STRINGTIMESERIESDATA = _descriptor.Descriptor(
  name='StringTimeseriesData',
  full_name='api.v2.StringTimeseriesData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='points', full_name='api.v2.StringTimeseriesData.points', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=147,
  serialized_end=210,
)


_NUMERICTIMESERIESDATA = _descriptor.Descriptor(
  name='NumericTimeseriesData',
  full_name='api.v2.NumericTimeseriesData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='points', full_name='api.v2.NumericTimeseriesData.points', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=212,
  serialized_end=277,
)


_TIMESERIESDATA = _descriptor.Descriptor(
  name='TimeseriesData',
  full_name='api.v2.TimeseriesData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='stringData', full_name='api.v2.TimeseriesData.stringData', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='numericData', full_name='api.v2.TimeseriesData.numericData', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data', full_name='api.v2.TimeseriesData.data',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=280,
  serialized_end=410,
)


_NAMEDTIMESERIESDATA = _descriptor.Descriptor(
  name='NamedTimeseriesData',
  full_name='api.v2.NamedTimeseriesData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='api.v2.NamedTimeseriesData.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stringData', full_name='api.v2.NamedTimeseriesData.stringData', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='numericData', full_name='api.v2.NamedTimeseriesData.numericData', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data', full_name='api.v2.NamedTimeseriesData.data',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=413,
  serialized_end=562,
)


_MULTINAMEDTIMESERIESDATA = _descriptor.Descriptor(
  name='MultiNamedTimeseriesData',
  full_name='api.v2.MultiNamedTimeseriesData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='namedTimeseriesData', full_name='api.v2.MultiNamedTimeseriesData.namedTimeseriesData', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=564,
  serialized_end=648,
)

_STRINGTIMESERIESDATA.fields_by_name['points'].message_type = _STRINGDATAPOINT
_NUMERICTIMESERIESDATA.fields_by_name['points'].message_type = _NUMERICDATAPOINT
_TIMESERIESDATA.fields_by_name['stringData'].message_type = _STRINGTIMESERIESDATA
_TIMESERIESDATA.fields_by_name['numericData'].message_type = _NUMERICTIMESERIESDATA
_TIMESERIESDATA.oneofs_by_name['data'].fields.append(
  _TIMESERIESDATA.fields_by_name['stringData'])
_TIMESERIESDATA.fields_by_name['stringData'].containing_oneof = _TIMESERIESDATA.oneofs_by_name['data']
_TIMESERIESDATA.oneofs_by_name['data'].fields.append(
  _TIMESERIESDATA.fields_by_name['numericData'])
_TIMESERIESDATA.fields_by_name['numericData'].containing_oneof = _TIMESERIESDATA.oneofs_by_name['data']
_NAMEDTIMESERIESDATA.fields_by_name['stringData'].message_type = _STRINGTIMESERIESDATA
_NAMEDTIMESERIESDATA.fields_by_name['numericData'].message_type = _NUMERICTIMESERIESDATA
_NAMEDTIMESERIESDATA.oneofs_by_name['data'].fields.append(
  _NAMEDTIMESERIESDATA.fields_by_name['stringData'])
_NAMEDTIMESERIESDATA.fields_by_name['stringData'].containing_oneof = _NAMEDTIMESERIESDATA.oneofs_by_name['data']
_NAMEDTIMESERIESDATA.oneofs_by_name['data'].fields.append(
  _NAMEDTIMESERIESDATA.fields_by_name['numericData'])
_NAMEDTIMESERIESDATA.fields_by_name['numericData'].containing_oneof = _NAMEDTIMESERIESDATA.oneofs_by_name['data']
_MULTINAMEDTIMESERIESDATA.fields_by_name['namedTimeseriesData'].message_type = _NAMEDTIMESERIESDATA
DESCRIPTOR.message_types_by_name['StringDatapoint'] = _STRINGDATAPOINT
DESCRIPTOR.message_types_by_name['NumericDatapoint'] = _NUMERICDATAPOINT
DESCRIPTOR.message_types_by_name['StringTimeseriesData'] = _STRINGTIMESERIESDATA
DESCRIPTOR.message_types_by_name['NumericTimeseriesData'] = _NUMERICTIMESERIESDATA
DESCRIPTOR.message_types_by_name['TimeseriesData'] = _TIMESERIESDATA
DESCRIPTOR.message_types_by_name['NamedTimeseriesData'] = _NAMEDTIMESERIESDATA
DESCRIPTOR.message_types_by_name['MultiNamedTimeseriesData'] = _MULTINAMEDTIMESERIESDATA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

StringDatapoint = _reflection.GeneratedProtocolMessageType('StringDatapoint', (_message.Message,), dict(
  DESCRIPTOR = _STRINGDATAPOINT,
  __module__ = 'api_timeseries_data_v2_pb2'
  # @@protoc_insertion_point(class_scope:api.v2.StringDatapoint)
  ))
_sym_db.RegisterMessage(StringDatapoint)

NumericDatapoint = _reflection.GeneratedProtocolMessageType('NumericDatapoint', (_message.Message,), dict(
  DESCRIPTOR = _NUMERICDATAPOINT,
  __module__ = 'api_timeseries_data_v2_pb2'
  # @@protoc_insertion_point(class_scope:api.v2.NumericDatapoint)
  ))
_sym_db.RegisterMessage(NumericDatapoint)

StringTimeseriesData = _reflection.GeneratedProtocolMessageType('StringTimeseriesData', (_message.Message,), dict(
  DESCRIPTOR = _STRINGTIMESERIESDATA,
  __module__ = 'api_timeseries_data_v2_pb2'
  # @@protoc_insertion_point(class_scope:api.v2.StringTimeseriesData)
  ))
_sym_db.RegisterMessage(StringTimeseriesData)

NumericTimeseriesData = _reflection.GeneratedProtocolMessageType('NumericTimeseriesData', (_message.Message,), dict(
  DESCRIPTOR = _NUMERICTIMESERIESDATA,
  __module__ = 'api_timeseries_data_v2_pb2'
  # @@protoc_insertion_point(class_scope:api.v2.NumericTimeseriesData)
  ))
_sym_db.RegisterMessage(NumericTimeseriesData)

TimeseriesData = _reflection.GeneratedProtocolMessageType('TimeseriesData', (_message.Message,), dict(
  DESCRIPTOR = _TIMESERIESDATA,
  __module__ = 'api_timeseries_data_v2_pb2'
  # @@protoc_insertion_point(class_scope:api.v2.TimeseriesData)
  ))
_sym_db.RegisterMessage(TimeseriesData)

NamedTimeseriesData = _reflection.GeneratedProtocolMessageType('NamedTimeseriesData', (_message.Message,), dict(
  DESCRIPTOR = _NAMEDTIMESERIESDATA,
  __module__ = 'api_timeseries_data_v2_pb2'
  # @@protoc_insertion_point(class_scope:api.v2.NamedTimeseriesData)
  ))
_sym_db.RegisterMessage(NamedTimeseriesData)

MultiNamedTimeseriesData = _reflection.GeneratedProtocolMessageType('MultiNamedTimeseriesData', (_message.Message,), dict(
  DESCRIPTOR = _MULTINAMEDTIMESERIESDATA,
  __module__ = 'api_timeseries_data_v2_pb2'
  # @@protoc_insertion_point(class_scope:api.v2.MultiNamedTimeseriesData)
  ))
_sym_db.RegisterMessage(MultiNamedTimeseriesData)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\027com.cognite.data.api.v2P\001\252\002\023Cognite.Data.Api.V2'))
# @@protoc_insertion_point(module_scope)



from setuptools import setup, find_packages

import cognite

packages = find_packages()
packages.remove('tests')

setup(name='cognite-sdk',
      version=cognite.__version__,
      description='Cognite API SDK for Python',
      url='http://cognite-sdk-python.readthedocs.io/',
      download_url='https://github.com/cognitedata/cognite-sdk-python/archive/{}.tar.gz'.format(cognite.__version__),
      author='Erlend Vollset',
      author_email='erlend.vollset@cognite.com',
      packages=packages,
      install_requires=[
          'requests',
          'pandas',
          'protobuf'
      ],
      zip_safe=False,
      include_package_data=True)



import numpy as np
import pandas as pd
import pytest

from cognite import _utils
from cognite.v05 import events, dto


@pytest.fixture(scope='module')
def get_post_event_obj():
    event = dto.Event(start_time=1521500400000, end_time=1521586800000)
    res = events.post_events([event])
    yield res
    res = events.get_events()
    ids = list(ev['id'] for ev in res.to_json())
    events.delete_events(ids)


def test_post_events(get_post_event_obj):
    assert isinstance(get_post_event_obj, dto.EventListResponse)
    assert isinstance(get_post_event_obj.to_pandas(), pd.DataFrame)
    assert isinstance(get_post_event_obj.to_json(), list)
    assert isinstance(get_post_event_obj.to_ndarray(), np.ndarray)


def test_post_events_length(get_post_event_obj):
    assert len(get_post_event_obj.to_json()) == 1


def test_get_event(get_post_event_obj):
    id = get_post_event_obj.to_json()[0]['id']
    res = events.get_event(event_id=id)
    assert isinstance(res, dto.EventResponse)
    assert isinstance(res.to_pandas(), pd.DataFrame)
    assert isinstance(res.to_json(), dict)
    assert isinstance(res.to_ndarray(), np.ndarray)


def test_get_event_invalid_id():
    with pytest.raises(_utils.APIError):
        events.get_event(123456789)


def test_get_events():
    res = events.get_events(min_start_time=1521500399999, max_start_time=1521500400001)
    assert isinstance(res, dto.EventListResponse)
    assert isinstance(res.to_pandas(), pd.DataFrame)
    assert isinstance(res.to_json(), list)
    assert isinstance(res.to_ndarray(), np.ndarray)
    for event in res:
        assert isinstance(event, dto.EventResponse)


def test_get_events_empty():
    res = events.get_events(max_start_time=1)
    assert res.to_pandas().empty
    assert len(res.to_json()) == 0


@pytest.fixture()
def post_event():
    event = dto.Event(start_time=1521500400000, end_time=1521586800000)
    res = events.post_events([event])
    return res


def test_delete_event(post_event):
    id = post_event.to_json()[0]['id']
    res = events.delete_events([id])
    assert res == {}



from random import randint

import numpy as np
import pandas as pd
import pytest

from cognite._utils import APIError
from cognite.v04 import raw
from cognite.v04.dto import RawResponse, RawRow

DB_NAME = None
TABLE_NAME = None
ROW_KEY = None
ROW_COLUMNS = None


@pytest.fixture(autouse=True, scope='class')
def db_name():
    global DB_NAME
    DB_NAME = 'test_db_{}'.format(randint(1, 2 ** 53 - 1))


@pytest.fixture(autouse=True, scope='class')
def table_name():
    global TABLE_NAME
    TABLE_NAME = 'test_table_{}'.format(randint(1, 2 ** 53 - 1))


@pytest.fixture(autouse=True, scope='class')
def row_key():
    global ROW_KEY
    ROW_KEY = 'test_key_{}'.format(randint(1, 2 ** 53 - 1))


@pytest.fixture(autouse=True, scope='class')
def row_columns():
    global ROW_COLUMNS
    ROW_COLUMNS = {'col1': 'val1'}


class TestDatabases:
    @pytest.fixture(scope='class')
    def databases(self):
        yield raw.get_databases()

    def test_create_databases(self):
        response = raw.create_databases([DB_NAME])
        assert isinstance(response, RawResponse)
        assert response.to_json()[0]['dbName'] == DB_NAME

    def test_databases_response_length(self, databases):
        assert len(databases.to_json()) > 0

    def test_get_databases_output_formats(self, databases):
        assert isinstance(databases, RawResponse)
        assert isinstance(databases.to_json(), list)
        assert isinstance(databases.to_ndarray(), np.ndarray)
        assert isinstance(databases.to_pandas(), pd.DataFrame)

    def test_delete_databases(self):
        response = raw.delete_databases([DB_NAME], recursive=True)
        assert response == {}
        with pytest.raises(APIError) as e:
            raw.delete_databases([DB_NAME])


class TestTables:
    @pytest.fixture(autouse=True, scope='class')
    def create_database(self):
        raw.create_databases([DB_NAME])
        yield
        raw.delete_databases([DB_NAME], recursive=True)

    @pytest.fixture(scope='class')
    def tables(self):
        yield raw.get_tables(DB_NAME)

    def test_create_tables(self):
        response = raw.create_tables(DB_NAME, [TABLE_NAME])
        # assert isinstance(response, RawObject)
        assert response.to_json()[0]['tableName'] == TABLE_NAME

    def test_tables_response_length(self, tables):
        assert len(tables.to_json()) > 0

    def test_tables_object_output_formats(self, tables):
        assert isinstance(tables, RawResponse)
        assert isinstance(tables.to_json(), list)
        assert isinstance(tables.to_ndarray(), np.ndarray)
        assert isinstance(tables.to_pandas(), pd.DataFrame)

    def test_delete_tables(self):
        response = raw.delete_tables(database_name=DB_NAME, table_names=[TABLE_NAME])
        assert response == {}
        with pytest.raises(APIError) as e:
            raw.delete_tables(DB_NAME, [TABLE_NAME])
            # assert re.match("{'code': 404, 'message': 'Did not find any dbs with the given names'}", str(e.value))
            # assert re.match("{'code': 404, 'message': 'No tables named test_table'}")


class TestRows:
    @pytest.fixture(autouse=True, scope='class')
    def create_database(self):
        raw.create_databases([DB_NAME])
        raw.create_tables(DB_NAME, [TABLE_NAME])
        yield
        raw.delete_databases([DB_NAME], recursive=True)

    def test_create_rows(self):
        response = raw.create_rows(DB_NAME, TABLE_NAME, rows=[RawRow(key=ROW_KEY, columns=ROW_COLUMNS)])
        assert response == {}

    def test_rows_response_length(self):
        rows = raw.get_rows(database_name=DB_NAME, table_name=TABLE_NAME).to_json()
        assert len(rows) == 1

    def test_rows_object_output_formats(self):
        row = raw.get_row(DB_NAME, TABLE_NAME, ROW_KEY)
        assert isinstance(row, RawResponse)
        assert isinstance(row.to_json(), list)
        assert isinstance(row.to_ndarray(), np.ndarray)
        assert isinstance(row.to_pandas(), pd.DataFrame)

    def test_delete_rows(self):
        response = raw.delete_rows(DB_NAME, TABLE_NAME, [RawRow(key=ROW_KEY, columns=ROW_COLUMNS)])
        assert response == {}




'Timeseries Module\n\nThis module mirrors the Timeseries API. It allows you to fetch data from the api and output it in various formats.\n\nhttps://doc.cognitedata.com/0.4/#Cognite-API-Time-series\n'
import io
import os
import warnings
from functools import partial
from multiprocessing import Pool
from urllib.parse import quote_plus
import pandas as pd
import cognite._constants as _constants
import cognite._utils as _utils
import cognite.config as config
from cognite._protobuf_descriptors import _api_timeseries_data_v1_pb2
from cognite.v04.dto import DatapointsResponse, DatapointsResponseIterator, LatestDatapointResponse, Datapoint, TimeSeries, TimeSeriesResponse, TimeseriesWithDatapoints

def get_datapoints(tag_id, aggregates=None, granularity=None, start=None, end=None, **kwargs):
    "Returns a DatapointsObject containing a list of datapoints for the given query.\n\n    This method will automate paging for the user and return all data for the given time period.\n\n    Args:\n        tag_id (str):           The tag_id to retrieve data for.\n\n        aggregates (list):      The list of aggregate functions you wish to apply to the data. Valid aggregate functions\n                                are: 'average/avg, max, min, count, sum, interpolation/int, stepinterpolation/step'.\n\n        granularity (str):      The granularity of the aggregate values. Valid entries are : 'day/d, hour/h, minute/m,\n                                second/s', or a multiple of these indicated by a number as a prefix e.g. '12hour'.\n\n        start (Union[str, int, datetime]):    Get datapoints after this time. Format is N[timeunit]-ago where timeunit is w,d,h,m,s.\n                                    E.g. '2d-ago' will get everything that is up to 2 days old. Can also send time in ms since\n                                    epoch or a datetime object which will be converted to ms since epoch UTC.\n\n        end (Union[str, int, datetime]):      Get datapoints up to this time. Same format as for start.\n\n    Keyword Arguments:\n        protobuf (bool):        Download the data using the binary protobuf format. Only applicable when getting raw data.\n                                Defaults to True.\n\n        processes (int):        Number of download processes to run in parallell. Defaults to number returned by cpu_count().\n\n        api_key (str):          Your api-key.\n\n        project (str):          Project name.\n\n    Returns:\n        v04.dto.DatapointsResponse: A data object containing the requested data with several getter methods with different\n        output formats.\n    "
    (api_key, project) = config.get_config_variables(kwargs.get('api_key'), kwargs.get('project'))
    (start, end) = _utils.interval_to_ms(start, end)
    diff = (end - start)
    num_of_processes = kwargs.get('processes', os.cpu_count())
    granularity_ms = 1
    if granularity:
        granularity_ms = _utils.granularity_to_ms(granularity)
    steps = min(num_of_processes, max(1, int((diff / granularity_ms))))
    step_size = _utils.round_to_nearest(int((diff / steps)), base=granularity_ms)
    step_starts = [(start + (i * step_size)) for i in range(steps)]
    args = [{'start': start, 'end': (start + step_size)} for start in step_starts]
    partial_get_dps = partial(_get_datapoints_helper_wrapper, tag_id=tag_id, aggregates=aggregates, granularity=granularity, protobuf=kwargs.get('protobuf', True), api_key=api_key, project=project)
    if (steps == 1):
        dps = _get_datapoints_helper(tag_id, aggregates, granularity, start, end, protobuf=kwargs.get('protobuf', True), api_key=api_key, project=project)
        return DatapointsResponse({'data': {'items': [{'tagId': tag_id, 'datapoints': dps}]}})
    prog_ind = _utils.ProgressIndicator([tag_id])
    p = Pool(steps)
    datapoints = p.map(partial_get_dps, args)
    p.close()
    p.join()
    concat_dps = []
    [concat_dps.extend(el) for el in datapoints]
    prog_ind.terminate()
    return DatapointsResponse({'data': {'items': [{'tagId': tag_id, 'datapoints': concat_dps}]}})

def _get_datapoints_helper_wrapper(args, tag_id, aggregates, granularity, protobuf, api_key, project):
    return _get_datapoints_helper(tag_id, aggregates, granularity, args['start'], args['end'], protobuf=protobuf, api_key=api_key, project=project)

def _get_datapoints_helper(tag_id, aggregates=None, granularity=None, start=None, end=None, **kwargs):
    "Returns a list of datapoints for the given query.\n\n    This method will automate paging for the given time period.\n\n    Args:\n        tag_id (str):           The tag_id to retrieve data for.\n\n        aggregates (list):      The list of aggregate functions you wish to apply to the data. Valid aggregate functions\n                                are: 'average/avg, max, min, count, sum, interpolation/int, stepinterpolation/step'.\n\n        granularity (str):      The granularity of the aggregate values. Valid entries are : 'day/d, hour/h, minute/m,\n                                second/s', or a multiple of these indicated by a number as a prefix e.g. '12hour'.\n\n        start (Union[str, int, datetime]):    Get datapoints after this time. Format is N[timeunit]-ago where timeunit is w,d,h,m,s.\n                                    E.g. '2d-ago' will get everything that is up to 2 days old. Can also send time in ms since\n                                    epoch or a datetime object which will be converted to ms since epoch UTC.\n\n        end (Union[str, int, datetime]):      Get datapoints up to this time. Same format as for start.\n\n    Keyword Arguments:\n        protobuf (bool):        Download the data using the binary protobuf format. Only applicable when getting raw data.\n                                Defaults to True.\n\n        api_key (str):          Your api-key.\n\n        project (str):          Project name.\n\n    Returns:\n        list of datapoints: A list containing datapoint dicts.\n    "
    (api_key, project) = (kwargs.get('api_key'), kwargs.get('project'))
    url = (config.get_base_url(api_version=0.4) + '/projects/{}/timeseries/data/{}'.format(project, quote_plus(tag_id)))
    use_protobuf = (kwargs.get('protobuf', True) and (aggregates is None))
    limit = (_constants.LIMIT if (aggregates is None) else _constants.LIMIT_AGG)
    params = {'aggregates': aggregates, 'granularity': granularity, 'limit': limit, 'start': start, 'end': end}
    headers = {'api-key': api_key, 'accept': ('application/protobuf' if use_protobuf else 'application/json')}
    datapoints = []
    while (((not datapoints) or (len(datapoints[(- 1)]) == limit)) and (params['end'] > params['start'])):
        res = _utils.get_request(url, params=params, headers=headers)
        if use_protobuf:
            ts_data = _api_timeseries_data_v1_pb2.TimeseriesData()
            ts_data.ParseFromString(res.content)
            res = [{'timestamp': p.timestamp, 'value': p.value} for p in ts_data.numericData.points]
        else:
            res = res.json()['data']['items'][0]['datapoints']
        if (not res):
            warning = 'An interval with no data has been requested ({}, {}).'.format(params['start'], params['end'])
            warnings.warn(warning)
            break
        datapoints.append(res)
        latest_timestamp = int(datapoints[(- 1)][(- 1)]['timestamp'])
        params['start'] = (latest_timestamp + (_utils.granularity_to_ms(granularity) if granularity else 1))
    dps = []
    [dps.extend(el) for el in datapoints]
    return dps

def _split_TimeseriesWithDatapoints_if_over_limit(timeseries_with_datapoints, limit):
    'Takes a TimeseriesWithDatapoints and splits it into multiple so that each has a max number of datapoints equal\n    to the limit given.\n\n    Args:\n        timeseries_with_datapoints (TimeseriesWithDatapoints): The timeseries with data to potentially split up.\n\n    Returns:\n        A list of TimeSeriesWithDatapoints where each has a maximum number of datapoints equal to the limit given.\n    '
    timeseries_with_datapoints_list = []
    if (len(timeseries_with_datapoints.datapoints) > limit):
        i = 0
        while (i < len(timeseries_with_datapoints.datapoints)):
            timeseries_with_datapoints_list.append(TimeseriesWithDatapoints(tagId=timeseries_with_datapoints.tagId, datapoints=timeseries_with_datapoints.datapoints[i:(i + limit)]))
            i += limit
    else:
        timeseries_with_datapoints_list.append(timeseries_with_datapoints)
    return timeseries_with_datapoints_list

def post_multi_tag_datapoints(timeseries_with_datapoints, **kwargs):
    'Insert data into multiple timeseries.\n\n    Args:\n        timeseries_with_datapoints (List[v04.dto.TimeseriesWithDatapoints]): The timeseries with data to insert.\n\n    Keyword Args:\n        api_key (str): Your api-key.\n\n        project (str): Project name.\n\n    Returns:\n        An empty response.\n    '
    (api_key, project) = config.get_config_variables(kwargs.get('api_key'), kwargs.get('project'))
    url = (config.get_base_url(api_version=0.4) + '/projects/{}/timeseries/data'.format(project))
    headers = {'api-key': api_key, 'content-type': 'application/json', 'accept': 'application/json'}
    ul_dps_limit = 100000
    timeseries_with_datapoints_limited = []
    for entry in timeseries_with_datapoints:
        timeseries_with_datapoints_limited.extend(_split_TimeseriesWithDatapoints_if_over_limit(entry, ul_dps_limit))
    timeseries_to_upload_binned = _utils.first_fit(list_items=timeseries_with_datapoints_limited, max_size=ul_dps_limit, get_count=(lambda x: len(x.datapoints)))
    for bin in timeseries_to_upload_binned:
        body = {'items': [{'tagId': ts_with_data.tagId, 'datapoints': [dp.__dict__ for dp in ts_with_data.datapoints]} for ts_with_data in bin]}
        res = _utils.post_request(url, body=body, headers=headers)
    return res.json()

def post_datapoints(tag_id, datapoints, **kwargs):
    'Insert a list of datapoints.\n\n    Args:\n        tag_id (str):       ID of timeseries to insert to.\n\n        datapoints (list[v04.dto.Datapoint): List of datapoint data transfer objects to insert.\n\n    Keyword Args:\n        api_key (str): Your api-key.\n\n        project (str): Project name.\n\n    Returns:\n        An empty response.\n    '
    (api_key, project) = config.get_config_variables(kwargs.get('api_key'), kwargs.get('project'))
    url = (config.get_base_url(api_version=0.4) + '/projects/{}/timeseries/data/{}'.format(project, quote_plus(tag_id)))
    headers = {'api-key': api_key, 'content-type': 'application/json', 'accept': 'application/json'}
    ul_dps_limit = 100000
    i = 0
    while (i < len(datapoints)):
        body = {'items': [dp.__dict__ for dp in datapoints[i:(i + ul_dps_limit)]]}
        res = _utils.post_request(url, body=body, headers=headers)
        i += ul_dps_limit
    return res.json()

def get_latest(tag_id, **kwargs):
    'Returns a LatestDatapointObject containing the latest datapoint for the given tag_id.\n\n    Args:\n        tag_id (str):           The tag_id to retrieve data for.\n\n    Keyword Arguments:\n        api_key (str):          Your api-key.\n\n        project (str):          Project name.\n\n    Returns:\n        v04.dto.LatestDatapointsResponse: A data object containing the requested data with several getter methods with different\n        output formats.\n    '
    (api_key, project) = config.get_config_variables(kwargs.get('api_key'), kwargs.get('project'))
    url = (config.get_base_url(api_version=0.4) + '/projects/{}/timeseries/latest/{}'.format(project, quote_plus(tag_id)))
    headers = {'api-key': api_key, 'accept': 'application/json'}
    res = _utils.get_request(url, headers=headers, cookies=config.get_cookies())
    return LatestDatapointResponse(res.json())

def get_multi_tag_datapoints(datapoints_queries, aggregates=None, granularity=None, start=None, end=None, **kwargs):
    "Returns a list of DatapointsObjects each of which contains a list of datapoints for the given timeseries.\n\n    This method will automate paging for the user and return all data for the given time period(s).\n\n    Args:\n        datapoints_queries (list[v04.dto.DatapointsQuery]): The list of DatapointsQuery objects specifying which\n                                                                    timeseries to retrieve data for.\n\n        aggregates (list, optional):    The list of aggregate functions you wish to apply to the data. Valid aggregate\n                                        functions are: 'average/avg, max, min, count, sum, interpolation/int,\n                                        stepinterpolation/step'.\n\n        granularity (str):              The granularity of the aggregate values. Valid entries are : 'day/d, hour/h,\n                                        minute/m, second/s', or a multiple of these indicated by a number as a prefix\n                                        e.g. '12hour'.\n\n        start (Union[str, int, datetime]):    Get datapoints after this time. Format is N[timeunit]-ago where timeunit is w,d,h,m,s.\n                                    E.g. '2d-ago' will get everything that is up to 2 days old. Can also send time in ms since\n                                    epoch or a datetime object which will be converted to ms since epoch UTC.\n\n        end (Union[str, int, datetime]):      Get datapoints up to this time. Same format as for start.\n\n    Keyword Arguments:\n        api_key (str):                  Your api-key.\n\n        project (str):                  Project name.\n\n    Returns:\n        list(v04.dto.DatapointsResponse): A list of data objects containing the requested data with several getter methods\n        with different output formats.\n    "
    (api_key, project) = config.get_config_variables(kwargs.get('api_key'), kwargs.get('project'))
    url = (config.get_base_url(api_version=0.4) + '/projects/{}/timeseries/dataquery'.format(project))
    (start, end) = _utils.interval_to_ms(start, end)
    num_of_dpqs_with_agg = 0
    num_of_dpqs_raw = 0
    for dpq in datapoints_queries:
        if (((dpq.aggregateFunctions is None) and (aggregates is None)) or (dpq.aggregateFunctions == '')):
            num_of_dpqs_raw += 1
        else:
            num_of_dpqs_with_agg += 1
    items = []
    for dpq in datapoints_queries:
        if ((dpq.aggregateFunctions is None) and (aggregates is None)):
            dpq.limit = int((_constants.LIMIT / num_of_dpqs_raw))
        else:
            dpq.limit = int((_constants.LIMIT_AGG / num_of_dpqs_with_agg))
        items.append(dpq.__dict__)
    body = {'items': items, 'aggregateFunctions': (','.join(aggregates) if (aggregates is not None) else None), 'granularity': granularity, 'start': start, 'end': end}
    headers = {'api-key': api_key, 'content-type': 'application/json', 'accept': 'application/json'}
    datapoints_responses = []
    has_incomplete_requests = True
    while has_incomplete_requests:
        res = _utils.post_request(url=url, body=body, headers=headers, cookies=config.get_cookies()).json()['data']['items']
        datapoints_responses.append(res)
        has_incomplete_requests = False
        for (i, dpr) in enumerate(res):
            dpq = datapoints_queries[i]
            if (len(dpr['datapoints']) == dpq.limit):
                has_incomplete_requests = True
                latest_timestamp = dpr['datapoints'][(- 1)]['timestamp']
                ts_granularity = (granularity if (dpq.granularity is None) else dpq.granularity)
                next_start = (latest_timestamp + (_utils.granularity_to_ms(ts_granularity) if ts_granularity else 1))
            else:
                next_start = (end - 1)
                if datapoints_queries[i].end:
                    next_start = (datapoints_queries[i].end - 1)
            datapoints_queries[i].start = next_start
    results = [{'data': {'items': [{'tagId': dpq.tagId, 'datapoints': []}]}} for dpq in datapoints_queries]
    for res in datapoints_responses:
        for (i, ts) in enumerate(res):
            results[i]['data']['items'][0]['datapoints'].extend(ts['datapoints'])
    return DatapointsResponseIterator([DatapointsResponse(result) for result in results])

def get_datapoints_frame(tag_ids, aggregates, granularity, start=None, end=None, **kwargs):
    "Returns a pandas dataframe of datapoints for the given tag_ids all on the same timestamps.\n\n    This method will automate paging for the user and return all data for the given time period.\n\n    Args:\n        tag_ids (list):     The list of tag_ids to retrieve data for. Each tag_id can be either a string containing the\n                            tag_id or a dictionary containing the tag_id and a list of specific aggregate functions.\n\n        aggregates (list):  The list of aggregate functions you wish to apply to the data for which you have not\n                            specified an aggregate function. Valid aggregate functions are: 'average/avg, max, min,\n                            count, sum, interpolation/int, stepinterpolation/step'.\n\n        granularity (str):  The granularity of the aggregate values. Valid entries are : 'day/d, hour/h, minute/m,\n                            second/s', or a multiple of these indicated by a number as a prefix e.g. '12hour'.\n\n        start (Union[str, int, datetime]):    Get datapoints after this time. Format is N[timeunit]-ago where timeunit is w,d,h,m,s.\n                                    E.g. '2d-ago' will get everything that is up to 2 days old. Can also send time in ms since\n                                    epoch or a datetime object which will be converted to ms since epoch UTC.\n\n        end (Union[str, int, datetime]):      Get datapoints up to this time. Same format as for start.\n\n    Keyword Arguments:\n        api_key (str): Your api-key.\n\n        project (str): Project name.\n\n        processes (int):    Number of download processes to run in parallell. Defaults to number returned by cpu_count().\n\n    Returns:\n        pandas.DataFrame: A pandas dataframe containing the datapoints for the given tag_ids. The datapoints for all the\n        tag_ids will all be on the same timestamps.\n\n    Note:\n        The ``tag_ids`` parameter can take a list of strings and/or dicts on the following formats::\n\n            Using strings:\n                ['<tag_id1>', '<tag_id2>']\n\n            Using dicts:\n                [{'tagId': '<tag_id1>', 'aggregates': ['<aggfunc1>', '<aggfunc2>']},\n                {'tagId': '<tag_id2>', 'aggregates': []}]\n\n            Using both:\n                ['<tagid1>', {'tagId': '<tag_id2>', 'aggregates': ['<aggfunc1>', '<aggfunc2>']}]\n    "
    (api_key, project) = config.get_config_variables(kwargs.get('api_key'), kwargs.get('project'))
    (start, end) = _utils.interval_to_ms(start, end)
    diff = (end - start)
    num_of_processes = kwargs.get('processes', os.cpu_count())
    granularity_ms = 1
    if granularity:
        granularity_ms = _utils.granularity_to_ms(granularity)
    steps = min(num_of_processes, max(1, int((diff / granularity_ms))))
    step_size = _utils.round_to_nearest(int((diff / steps)), base=granularity_ms)
    step_starts = [(start + (i * step_size)) for i in range(steps)]
    args = [{'start': start, 'end': (start + step_size)} for start in step_starts]
    partial_get_dpsf = partial(_get_datapoints_frame_helper_wrapper, tag_ids=tag_ids, aggregates=aggregates, granularity=granularity, api_key=api_key, project=project)
    if (steps == 1):
        return _get_datapoints_frame_helper(tag_ids, aggregates, granularity, start, end, api_key=api_key, project=project)
    prog_ind = _utils.ProgressIndicator(tag_ids)
    p = Pool(steps)
    dataframes = p.map(partial_get_dpsf, args)
    p.close()
    p.join()
    df = pd.concat(dataframes).drop_duplicates(subset='timestamp').reset_index(drop=True)
    prog_ind.terminate()
    return df

def _get_datapoints_frame_helper_wrapper(args, tag_ids, aggregates, granularity, api_key, project):
    return _get_datapoints_frame_helper(tag_ids, aggregates, granularity, args['start'], args['end'], api_key=api_key, project=project)

def _get_datapoints_frame_helper(tag_ids, aggregates, granularity, start=None, end=None, **kwargs):
    "Returns a pandas dataframe of datapoints for the given tag_ids all on the same timestamps.\n\n    This method will automate paging for the user and return all data for the given time period.\n\n    Args:\n        tag_ids (list):     The list of tag_ids to retrieve data for. Each tag_id can be either a string containing the\n                            tag_id or a dictionary containing the tag_id and a list of specific aggregate functions.\n\n        aggregates (list):  The list of aggregate functions you wish to apply to the data for which you have not\n                            specified an aggregate function. Valid aggregate functions are: 'average/avg, max, min,\n                            count, sum, interpolation/int, stepinterpolation/step'.\n\n        granularity (str):  The granularity of the aggregate values. Valid entries are : 'day/d, hour/h, minute/m,\n                            second/s', or a multiple of these indicated by a number as a prefix e.g. '12hour'.\n\n        start (Union[str, int, datetime]):    Get datapoints after this time. Format is N[timeunit]-ago where timeunit is w,d,h,m,s.\n                                    E.g. '2d-ago' will get everything that is up to 2 days old. Can also send time in ms since\n                                    epoch or a datetime object which will be converted to ms since epoch UTC.\n\n        end (Union[str, int, datetime]):      Get datapoints up to this time. Same format as for start.\n\n    Keyword Arguments:\n        api_key (str): Your api-key.\n\n        project (str): Project name.\n\n    Returns:\n        pandas.DataFrame: A pandas dataframe containing the datapoints for the given tag_ids. The datapoints for all the\n        tag_ids will all be on the same timestamps.\n\n    Note:\n        The ``tag_ids`` parameter can take a list of strings and/or dicts on the following formats::\n\n            Using strings:\n                ['<tag_id1>', '<tag_id2>']\n\n            Using dicts:\n                [{'tagId': '<tag_id1>', 'aggregates': ['<aggfunc1>', '<aggfunc2>']},\n                {'tagId': '<tag_id2>', 'aggregates': []}]\n\n            Using both:\n                ['<tagid1>', {'tagId': '<tag_id2>', 'aggregates': ['<aggfunc1>', '<aggfunc2>']}]\n    "
    (api_key, project) = (kwargs.get('api_key'), kwargs.get('project'))
    url = (config.get_base_url(api_version=0.4) + '/projects/{}/timeseries/dataframe'.format(project))
    num_aggregates = 0
    for tag in tag_ids:
        if (isinstance(tag, str) or (tag.get('aggregates') is None)):
            num_aggregates += len(aggregates)
        else:
            num_aggregates += len(tag['aggregates'])
    per_tag_limit = int((_constants.LIMIT / num_aggregates))
    body = {'items': [({'tagId': '{}'.format(tag_id)} if isinstance(tag_id, str) else {'tagId': '{}'.format(tag_id['tagId']), 'aggregates': tag_id.get('aggregates', [])}) for tag_id in tag_ids], 'aggregates': aggregates, 'granularity': granularity, 'start': start, 'end': end, 'limit': per_tag_limit}
    headers = {'api-key': api_key, 'content-type': 'application/json', 'accept': 'text/csv'}
    dataframes = []
    while (((not dataframes) or (dataframes[(- 1)].shape[0] == per_tag_limit)) and (body['end'] > body['start'])):
        res = _utils.post_request(url=url, body=body, headers=headers, cookies=config.get_cookies())
        dataframes.append(pd.read_csv(io.StringIO(res.content.decode((res.encoding if res.encoding else res.apparent_encoding)))))
        if dataframes[(- 1)].empty:
            warning = 'An interval with no data has been requested ({}, {}).'.format(body['start'], body['end'])
            warnings.warn(warning)
            break
        latest_timestamp = int(dataframes[(- 1)].iloc[((- 1), 0)])
        body['start'] = (latest_timestamp + _utils.granularity_to_ms(granularity))
    return pd.concat(dataframes).reset_index(drop=True)

def get_timeseries(prefix=None, description=None, include_metadata=False, asset_id=None, path=None, **kwargs):
    'Returns a TimeseriesObject containing the requested timeseries.\n\n    Args:\n        prefix (str):           List timeseries with this prefix in the name.\n\n        description (str):      Filter timeseries taht contains this string in its description.\n\n        include_metadata (bool):    Decide if the metadata field should be returned or not. Defaults to False.\n\n        asset_id (int):        Get timeseries related to this asset.\n\n        path (str):             Get timeseries under this asset path branch.\n\n    Keyword Arguments:\n        limit (int):            Number of results to return.\n\n        api_key (str):          Your api-key.\n\n        project (str):          Project name.\n\n        autopaging (bool):      Whether or not to automatically page through results. If set to true, limit will be\n                                disregarded. Defaults to False.\n\n    Returns:\n        v04.dto.TimeSeriesResponse: A data object containing the requested timeseries with several getter methods with different\n        output formats.\n    '
    (api_key, project) = config.get_config_variables(kwargs.get('api_key'), kwargs.get('project'))
    url = (config.get_base_url(api_version=0.4) + '/projects/{}/timeseries'.format(project))
    headers = {'api-key': api_key, 'accept': 'application/json'}
    params = {'q': prefix, 'description': description, 'includeMetadata': include_metadata, 'assetId': asset_id, 'path': path, 'limit': (kwargs.get('limit', 10000) if (not kwargs.get('autopaging')) else 10000)}
    timeseries = []
    res = _utils.get_request(url=url, headers=headers, params=params, cookies=config.get_cookies())
    timeseries.extend(res.json()['data']['items'])
    next_cursor = res.json()['data'].get('nextCursor')
    while (next_cursor and kwargs.get('autopaging')):
        params['cursor'] = next_cursor
        res = _utils.get_request(url=url, headers=headers, params=params, cookies=config.get_cookies())
        timeseries.extend(res.json()['data']['items'])
        next_cursor = res.json()['data'].get('nextCursor')
    return TimeSeriesResponse({'data': {'nextCursor': next_cursor, 'previousCursor': res.json()['data'].get('previousCursor'), 'items': timeseries}})

def post_time_series(time_series, **kwargs):
    'Create a new time series.\n\n    Args:\n        timeseries (list[v04.dto.TimeSeries]):   List of time series data transfer objects to create.\n\n    Keyword Args:\n        api_key (str): Your api-key.\n\n        project (str): Project name.\n    Returns:\n        An empty response.\n    '
    (api_key, project) = config.get_config_variables(kwargs.get('api_key'), kwargs.get('project'))
    url = (config.get_base_url(api_version=0.4) + '/projects/{}/timeseries'.format(project))
    body = {'items': [ts.__dict__ for ts in time_series]}
    headers = {'api-key': api_key, 'content-type': 'application/json', 'accept': 'application/json'}
    res = _utils.post_request(url, body=body, headers=headers)
    return res.json()

def update_time_series(time_series, **kwargs):
    'Update an existing time series.\n\n    For each field that can be updated, a null value indicates that nothing should be done.\n\n    Args:\n        timeseries (list[v04.dto.TimeSeries]):   List of time series data transfer objects to update.\n\n    Keyword Args:\n        api_key (str): Your api-key.\n\n        project (str): Project name.\n\n    Returns:\n        An empty response.\n    '
    (api_key, project) = config.get_config_variables(kwargs.get('api_key'), kwargs.get('project'))
    url = (config.get_base_url(api_version=0.4) + '/projects/{}/timeseries'.format(project))
    body = {'items': [ts.__dict__ for ts in time_series]}
    headers = {'api-key': api_key, 'content-type': 'application/json', 'accept': 'application/json'}
    res = _utils.put_request(url, body=body, headers=headers)
    return res.json()




'Cognite API SDK\n\nThis package has been created to ensure excellent excellent user experience for data scientists using the Cognite\nData Platform (CDP).\n'
__all__ = ['v04', 'v05', 'preprocessing', 'config']
__version__ = '0.8.1'




'Assets Module.\n\nThis module mirrors the Assets API.\n\nhttps://doc.cognitedata.com/0.5/#Cognite-API-Assets\n'
import cognite._constants as constants
import cognite._utils as utils
import cognite.config as config
from cognite.v05.dto import AssetResponse, AssetListResponse, Asset

def get_assets(name=None, path=None, description=None, metadata=None, depth=None, fuzziness=None, **kwargs):
    'Returns assets matching provided description.\n\n    Args:\n        name (str):             The name of the asset(s) to get.\n\n        path (str):             The path of the subtree to search in.\n\n        description (str):      Search query.\n\n        metadata (str):         The metadata values used to filter the results.\n\n        depth (int):            Get sub assets up oto this many levels below the specified path.\n\n        fuzziness (int):        The degree of fuzziness in the name matching.\n\n    Keyword Arguments:\n        limit (int):            The maximum number of assets to be returned.\n\n        cursor (str):           Cursor to use for paging through results.\n\n        api_key (str):          Your api-key.\n\n        project (str):          Project name.\n    Returns:\n        v05.dto.AssetResponse: A data object containing the requested assets with several getter methods with different\n        output formats.\n    '
    (api_key, project) = config.get_config_variables(kwargs.get('api_key'), kwargs.get('project'))
    url = (config.get_base_url(api_version=0.5) + '/projects/{}/assets'.format(project))
    params = {'name': name, 'description': description, 'path': path, 'metadata': metadata, 'depth': depth, 'fuzziness': fuzziness, 'cursor': kwargs.get('cursor'), 'limit': kwargs.get('limit', constants.LIMIT)}
    headers = {'api-key': api_key, 'accept': 'application/json'}
    res = utils.get_request(url, params=params, headers=headers, cookies=config.get_cookies())
    return AssetListResponse(res.json())

def get_asset(asset_id, **kwargs):
    'Returns the asset with the provided assetId.\n\n    Args:\n        asset_id (int):         The asset id of the top asset to get.\n\n    Keyword Arguments:\n        api_key (str):          Your api-key.\n\n        project (str):          Project name.\n    Returns:\n        v05.dto.AssetResponse: A data object containing the requested assets with several getter methods with different\n        output formats.\n    '
    (api_key, project) = config.get_config_variables(kwargs.get('api_key'), kwargs.get('project'))
    url = (config.get_base_url(api_version=0.5) + '/projects/{}/assets/{}/subtree'.format(project, asset_id))
    headers = {'api-key': api_key, 'accept': 'application/json'}
    res = utils.get_request(url, headers=headers, cookies=config.get_cookies())
    return AssetResponse(res.json())

def get_asset_subtree(asset_id, depth=None, **kwargs):
    'Returns asset subtree of asset with provided assetId.\n\n    Args:\n        asset_id (int):         The asset id of the top asset to get.\n\n        depth (int):            Get subassets this many levels below the top asset.\n\n    Keyword Arguments:\n        limit (int):            The maximum nuber of assets to be returned.\n\n        cursor (str):           Cursor to use for paging through results.\n\n        api_key (str):          Your api-key.\n\n        project (str):          Project name.\n    Returns:\n        v05.dto.AssetResponse: A data object containing the requested assets with several getter methods with different\n        output formats.\n    '
    (api_key, project) = config.get_config_variables(kwargs.get('api_key'), kwargs.get('project'))
    url = (config.get_base_url(api_version=0.5) + '/projects/{}/assets/{}/subtree'.format(project, asset_id))
    params = {'depth': depth, 'limit': kwargs.get('limit', constants.LIMIT), 'cursor': kwargs.get('cursor')}
    headers = {'api-key': api_key, 'accept': 'application/json'}
    res = utils.get_request(url, params=params, headers=headers, cookies=config.get_cookies())
    return AssetListResponse(res.json())

def post_assets(assets, **kwargs):
    'Insert a list of assets.\n\n    Args:\n        assets (list[v05.dto.Asset]): List of asset data transfer objects.\n\n    Keyword Args:\n        api_key (str): Your api-key.\n\n        project (str): Project name.\n\n    Returns:\n        v05.dto.AssetResponse: A data object containing the posted assets with several getter methods with different\n        output formats.\n    '
    (api_key, project) = config.get_config_variables(kwargs.get('api_key'), kwargs.get('project'))
    url = (config.get_base_url(api_version=0.5) + '/projects/{}/assets'.format(project))
    body = {'items': [asset.__dict__ for asset in assets]}
    headers = {'api-key': api_key, 'content-type': 'application/json', 'accept': 'application/json'}
    res = utils.post_request(url, body=body, headers=headers, cookies=config.get_cookies())
    return AssetListResponse(res.json())

def delete_assets(asset_ids, **kwargs):
    'Delete a list of assets.\n\n    Args:\n        asset_ids (list[int]): List of IDs of assets to delete.\n\n    Keyword Args:\n        api_key (str): Your api-key.\n\n        project (str): Project name.\n\n    Returns:\n        An empty response.\n    '
    (api_key, project) = config.get_config_variables(kwargs.get('api_key'), kwargs.get('project'))
    url = (config.get_base_url(api_version=0.5) + '/projects/{}/assets/delete'.format(project))
    body = {'items': asset_ids}
    headers = {'api-key': api_key, 'content-type': 'application/json', 'accept': 'application/json'}
    res = utils.post_request(url, body=body, headers=headers, cookies=config.get_cookies())
    return res.json()



import numpy as np
import pandas as pd
import pytest

from cognite.v05 import assets
from cognite.v05.dto import Asset, AssetResponse, AssetListResponse

ASSET_NAME = 'test_asset'


@pytest.fixture(scope='module')
def get_asset_subtree_response():
    return assets.get_asset_subtree(asset_id=6354653755843357, limit=1)


@pytest.fixture(scope='module')
def get_assets_response():
    return assets.get_assets(limit=1)


def test_get_assets_response_object(get_assets_response):
    assert isinstance(get_assets_response, AssetListResponse)
    assert get_assets_response.next_cursor() is not None
    assert get_assets_response.previous_cursor() is None


def test_get_asset():
    res = assets.get_asset(6354653755843357)
    assert isinstance(res, AssetResponse)
    assert isinstance(res.to_json(), dict)
    assert isinstance(res.to_pandas(), pd.DataFrame)
    assert isinstance(res.to_ndarray(), np.ndarray)


def test_asset_subtree_object(get_asset_subtree_response):
    assert isinstance(get_asset_subtree_response, AssetListResponse)
    assert get_asset_subtree_response.next_cursor() is not None
    assert get_asset_subtree_response.previous_cursor() is None


def test_json(get_asset_subtree_response):
    assert isinstance(get_asset_subtree_response.to_json(), list)


def test_pandas(get_asset_subtree_response):
    assert isinstance(get_asset_subtree_response.to_pandas(), pd.DataFrame)


def test_ndarray(get_asset_subtree_response):
    assert isinstance(get_asset_subtree_response.to_ndarray(), np.ndarray)


def test_post_assets():
    a1 = Asset(name=ASSET_NAME)
    res = assets.post_assets([a1])
    assert isinstance(res, AssetListResponse)
    assert res.to_json()[0]['name'] == ASSET_NAME
    assert res.to_json()[0].get('id') != None


def test_delete_assets():
    asset = assets.get_assets(ASSET_NAME, depth=0)
    id = asset.to_json()[0]['id']
    res = assets.delete_assets([id])
    assert res == {}
    assert len(assets.get_assets(ASSET_NAME, depth=0).to_json()) == 0
